from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class QueryStory(Query):
    """
    Class for obtaining available info for one or more stories (clusters) in the Event Registry
    NOTE: Story in our terminology is a cluster of articles (and not a single article). An event is
    then something that consists of one or more stories (typically in different languages).

    @param storyUriOrList: a single story uri or a list of story uris
    """
    def __init__(self, storyUriOrList = None):
        super(QueryStory, self).__init__()
        self._setVal("action", "getStory")
        if storyUriOrList != None:
            self.queryByUri(storyUriOrList)


    def _getPath(self):
        return "/json/story"


    def queryByUri(self, uriOrUriList):
        """search stories by their uri(s)"""
        self._setVal("storyUri", uriOrUriList)


    def addRequestedResult(self, requestStory):
        """
        Add a result type that you would like to be returned.
        In one QueryStory you can ask for multiple result types.
        Result types can be the classes that extend RequestStory base class (see classes below).
        """
        assert isinstance(requestStory, RequestStory), "QueryStory class can only accept result requests that are of type RequestStory"
        self.resultTypeList = [item for item in self.resultTypeList if item.getResultType() != requestStory.getResultType()]
        self.resultTypeList.append(requestStory)


    def setRequestedResult(self, requestStory):
        """
        Set the single result type that you would like to be returned. If some other request type was previously set, it will be overwritten.
        Result types can be the classes that extend RequestStory base class (see classes below).
        """
        assert isinstance(requestStory, RequestStory), "QueryStory class can only accept result requests that are of type RequestStory"
        self.resultTypeList = [requestStory]



class RequestStory:
    def __init__(self):
        self.resultType = None


    def getResultType(self):
        return self.resultType



class RequestStoryInfo(RequestStory):
    """
    return details about a story
    """
    def __init__(self, returnInfo = ReturnInfo()):
        self.resultType = "info"
        self.__dict__.update(returnInfo.getParams("info"))



class RequestStoryArticles(RequestStory):
    """
    return articles about the story
    """
    def __init__(self,
                 page = 1,
                 count = 20,
                 sortBy = "cosSim", sortByAsc = False,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 200))):
        """
        return articles in the story (cluster)
        @param page: page of the articles to return (1, 2, ...)
        @param count: number of articles to return per page (at most 200)
        @param sortBy: order in which articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to event centroid), sourceImportanceRank (importance of the news source, custom set), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares in social media)
        @param sortByAsc: should the articles be sorted in ascending order (True) or descending (False) based on sortBy value
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 200
        self.resultType = "articles"
        self.articlesPage = page
        self.articlesCount = count
        self.articlesSortBy = sortBy
        self.articlesSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("articles"))



class RequestStoryArticleUris(RequestStory):
    """
    return a list of article uris
    """
    def __init__(self,
                 sortBy = "cosSim", sortByAsc = False  # order in which story articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to story centroid), socialScore (total shares in social media), facebookShares (shares on fb), twitterShares (shares on twitter)
                 ):
        """
        return articles in the story (cluster)
        @param sortBy: order in which articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to event centroid), sourceImportanceRank (importance of the news source, custom set), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares in social media)
        @param sortByAsc: should the articles be sorted in ascending order (True) or descending (False) based on sortBy value
        """
        self.articleUrisSortBy = sortBy
        self.articleUrisSortByAsc = sortByAsc
        self.resultType = "articleUris"



class RequestStoryArticleTrend(RequestStory):
    """
    return trending information for the articles about the story
    """
    def __init__(self,
                 lang = mainLangs,
                 minArticleCosSim = -1,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 0))):
        self.resultType = "articleTrend"
        self.articleTrendLang = lang
        self.articleTrendMinArticleCosSim = minArticleCosSim
        self.__dict__.update(returnInfo.getParams("articleTrend"))



class RequestStorySimilarStories(RequestStory):
    """
    return a list of similar stories
    """
    def __init__(self,
                 count = 20,                    # number of similar stories to return
                 source = "concept",            # how to compute similarity. Options: concept cca
                 maxDayDiff = sys.maxsize,       # what is the maximum time difference between the similar stories and this one
                 returnInfo = ReturnInfo()):
        assert count <= 200
        self.resultType = "similarStories"
        self.similarEventsCount = count
        self.similarEventsSource = source
        if maxDayDiff != sys.maxsize:
            self.similarEventsMaxDayDiff = maxDayDiff
        self.similarEventsAddArticleTrendInfo = addArticleTrendInfo
        self.__dict__.update(returnInfo.getParams("similarEvents"))



"""
provides classes for obtaining information about how frequently individual concepts or categories
have been mentioned in news articles (if source == "news") of in social media (if source == "social")
"""

from eventregistry.Base import *
from eventregistry.ReturnInfo import *


class CountsBase(QueryParamsBase):
    def _getPath(self):
        return "/json/counters"



class GetCounts(CountsBase):
    def __init__(self,
                 uriOrUriList,
                 source = "news",
                 type = "concept",
                 startDate = None,
                 endDate = None,
                 returnInfo = ReturnInfo()):
        """
        obtain information about how frequently a concept or category is mentioned in the articles on particular dates
        by specifying source="custom" one can obtain counts for custom concepts, such as stocks, macroeconomic indicators, etc. The uri
        for these can be found using EventRegistry.getCustomConceptUri() method.
        Usage example:
            q = GetCounts([er.getConceptUri("Obama"), er.getConceptUri("ebola")])
            ret = er.execQuery(q)
        Return object:
            {
                "http://en.wikipedia.org/wiki/Barack_Obama": [
                    {
                        "count": 1,
                        "date": "2015-05-07"
                    },
                    {
                        "count": 4,
                        "date": "2015-05-08"
                    },
                    ...
                ],
                "http://en.wikipedia.org/wiki/Ebola_virus_disease": [
                    {
                        "count": 0,
                        "date": "2015-05-07"
                    },
                    ...
                ]
            }

        @param uriOrUriList: concept/category uri or a list of uris
        @param source: input source information from which to compute top trends. Options: "news", "social", "custom", "geo" or "sentiment"
        @param type: what do the uris represent? "concept" or "category"
        @param startDate: starting date from which to provide counts onwards (either None, datetime.date or "YYYY-MM-DD")
        @param endDate: ending date until which to provide counts (either None, datetime.date or "YYYY-MM-DD")
        @param returnInfo: what details should be included in the returned information
        """
        CountsBase.__init__(self)
        self._setVal("action", "getCounts")
        self._setVal("source", source)
        self._setVal("type", type)
        self._update(returnInfo.getParams())
        self._setVal("uri", uriOrUriList)
        if startDate != None:
            self._setDateVal("startDate", startDate)
        if endDate != None:
            self._setDateVal("endDate", endDate)



class GetCountsEx(CountsBase):
    def __init__(self,
                 uriOrUriList,
                 source = "news",
                 type = "concept",
                 startDate = None,
                 endDate = None,
                 returnInfo = ReturnInfo()):
        """
        obtain information about how frequently a concept or category is mentioned in the articles on particular dates
        Similar to GetCounts, but the output is more friendly for a larger set of provided uris/ids at once
        Usage example:
            q = GetCountsEx(type = "category")
            q.queryById(range(10))  # return trends of first 10 categories
            ret = er.execQuery(q)
        Return object:
            {
                "categoryInfo": [
                    {
                        "id": 0,
                        "label": "Root",
                        "uri": "http://www.dmoz.org"
                    },
                    {
                        "id": 1,
                        "label": "Recreation",
                        "uri": "http://www.dmoz.org/Recreation"
                    },
                    ...
                ],
                "counts": [
                    {
                        "0": 23, "1": 42, "2": 52, "3": 32, "4": 21, "5": 65, "6": 32, "7": 654, "8": 1, "9": 34,
                        "date": "2015-05-07"
                    },
                    ...
                ]
            }

        @param uriOrUriList: concept/category uri or a list of uris
        @param source: input source information from which to compute top trends. Options: "news", "social"
        @param type: what do the uris represent? "concept" or "category"
        @param startDate: starting date from which to provide counts onwards (either None, datetime.date or "YYYY-MM-DD")
        @param endDate: ending date until which to provide counts (either None, datetime.date or "YYYY-MM-DD")
        @param returnInfo: what details should be included in the returned information
        """
        CountsBase.__init__(self)
        self._setVal("action", "getCountsEx")
        self._setVal("source", source)
        self._setVal("type", type)
        self._update(returnInfo.getParams())
        self._setVal("uri", uriOrUriList)
        if startDate != None:
            self._setDateVal("startDate", startDate)
        if endDate != None:
            self._setDateVal("endDate", endDate)



import six, json
from eventregistry.Base import *
from eventregistry.ReturnInfo import *
from eventregistry.Query import *


class QueryEvents(Query):
    def __init__(self,
                 keywords = None,
                 conceptUri = None,
                 categoryUri = None,
                 sourceUri = None,
                 sourceLocationUri = None,
                 sourceGroupUri = None,
                 locationUri = None,
                 lang = None,
                 dateStart = None,
                 dateEnd = None,
                 minArticlesInEvent = 0,
                 maxArticlesInEvent = sys.maxsize,
                 dateMentionStart = None,
                 dateMentionEnd = None,
                 ignoreKeywords = None,
                 ignoreConceptUri = None,
                 ignoreCategoryUri = None,
                 ignoreSourceUri = None,
                 ignoreSourceLocationUri = None,
                 ignoreSourceGroupUri = None,
                 ignoreLocationUri = None,
                 ignoreLang = None,
                 keywordsLoc = "body",
                 ignoreKeywordsLoc = "body",
                 requestedResult = None):
        """
        Query class for searching for events in the Event Registry.
        The resulting events have to match all specified conditions. If a parameter value equals "" or [], then it is ignored.
        In order for query to be valid, it has to have at least one positive condition (condition that does not start with ignore*).

        @param keywords: find events where articles mention all the specified keywords.
            A single keyword/phrase can be provided as a string, multiple keywords/phrases can be provided as a list of strings.
            Use QueryItems.AND() if *all* provided keywords/phrases should be mentioned, or QueryItems.OR() if *any* of the keywords/phrases should be mentioned.
        @param conceptUri: find events where the concept with concept uri is important.
            A single concept uri can be provided as a string, multiple concept uris can be provided as a list of strings.
            Use QueryItems.AND() if *all* provided concepts should be mentioned, or QueryItems.OR() if *any* of the concepts should be mentioned.
            To obtain a concept uri using a concept label use EventRegistry.getConceptUri().
        @param categoryUri: find events that are assigned into a particular category.
            A single category uri can be provided as a string, multiple category uris can be provided as a list of strings.
            Use QueryItems.AND() if *all* provided categories should be mentioned, or QueryItems.OR() if *any* of the categories should be mentioned.
            A category uri can be obtained from a category name using EventRegistry.getCategoryUri().
        @param sourceUri: find events that contain one or more articles that have been written by a news source sourceUri.
            If multiple sources should be considered use QueryItems.OR() to provide the list of sources.
            Source uri for a given news source name can be obtained using EventRegistry.getNewsSourceUri().
        @param sourceLocationUri: find events that contain one or more articles that were written by news sources located in the given geographic location.
            If multiple source locations are provided, then put them into a list inside QueryItems.OR()
            Location uri can either be a city or a country. Location uri for a given name can be obtained using EventRegistry.getLocationUri().
        @param sourceGroupUri: find events that contain one or more articles that were written by news sources that are assigned to the specified source group.
            If multiple source groups are provided, then put them into a list inside QueryItems.OR()
            Source group uri for a given name can be obtained using EventRegistry.getSourceGroupUri().
        @param locationUri: find events that occured at a particular location.
            If value can be a string or a list of strings provided in QueryItems.OR().
            Location uri can either be a city or a country. Location uri for a given name can be obtained using EventRegistry.getLocationUri().
        @param lang: find events for which we found articles in the specified language.
            If more than one language is specified, resulting events has to be reported in *any* of the languages.
        @param dateStart: find events that occured on or after dateStart. Date should be provided in YYYY-MM-DD format, datetime.time or datetime.datetime.
        @param dateEnd: find events that occured before or on dateEnd. Date should be provided in YYYY-MM-DD format, datetime.time or datetime.datetime.
        @param minArticlesInEvent: find events that have been reported in at least minArticlesInEvent articles (regardless of language)
        @param maxArticlesInEvent: find events that have not been reported in more than maxArticlesInEvent articles (regardless of language)
        @param dateMentionStart: find events where articles explicitly mention a date that is equal or greater than dateMentionStart.
        @param dateMentionEnd: find events where articles explicitly mention a date that is lower or equal to dateMentionEnd.
        @param ignoreKeywords: ignore events where articles about the event mention any of the provided keywords
        @param ignoreConceptUri: ignore events that are about any of the provided concepts
        @param ignoreCategoryUri: ignore events that are about any of the provided categories
        @param ignoreSourceUri: ignore events that have have articles which have been written by any of the specified news sources
        @param ignoreSourceLocationUri: ignore events that have articles which been written by sources located at *any* of the specified locations
        @param ignoreSourceGroupUri: ignore events that have articles which have been written by sources in *any* of the specified source groups
        @param ignoreLocationUri: ignore events that occured in any of the provided locations. A location can be a city or a place
        @param ignoreLang: ignore events that are reported in any of the provided languages
        @param keywordsLoc: what data should be used when searching using the keywords provided by "keywords" parameter. "body" (default), "title", or "body,title"
        @param ignoreKeywordsLoc: what data should be used when searching using the keywords provided by "ignoreKeywords" parameter. "body" (default), "title", or "body,title"
        @param requestedResult: the information to return as the result of the query. By default return the list of matching events
        """
        super(QueryEvents, self).__init__()

        self._setVal("action", "getEvents")

        self._setQueryArrVal(keywords, "keyword", "keywordOper", "and")
        self._setQueryArrVal(conceptUri, "conceptUri", "conceptOper", "and")
        self._setQueryArrVal(categoryUri, "categoryUri", "categoryOper", "or")
        self._setQueryArrVal(sourceUri, "sourceUri", None, "or")
        self._setQueryArrVal(sourceLocationUri, "sourceLocationUri", None, "or")
        self._setQueryArrVal(sourceGroupUri, "sourceGroupUri", None, "or")
        self._setQueryArrVal(locationUri, "locationUri", None, "or")        # location such as "http://en.wikipedia.org/wiki/Ljubljana"

        self._setQueryArrVal(lang, "lang", None, "or")                      # a single lang or list (possible: eng, deu, spa, zho, slv)

        if (dateStart != None):
            self._setDateVal("dateStart", dateStart)   # 2014-05-02
        if (dateEnd != None):
            self._setDateVal("dateEnd", dateEnd)       # 2014-05-02

        self._setValIfNotDefault("minArticlesInEvent", minArticlesInEvent, 0)
        self._setValIfNotDefault("maxArticlesInEvent", maxArticlesInEvent, sys.maxsize)

        if (dateMentionStart != None):
            self._setDateVal("dateMentionStart", dateMentionStart)    # e.g. 2014-05-02
        if (dateMentionEnd != None):
            self._setDateVal("dateMentionEnd", dateMentionEnd)        # e.g. 2014-05-02

        # for the negative conditions, only the OR is a valid operator type
        self._setQueryArrVal(ignoreKeywords, "ignoreKeywords", None, "or")
        self._setQueryArrVal(ignoreConceptUri, "ignoreConceptUri", None, "or")
        self._setQueryArrVal(ignoreCategoryUri, "ignoreCategoryUri", None, "or")
        self._setQueryArrVal(ignoreSourceUri, "ignoreSourceUri", None, "or")
        self._setQueryArrVal(ignoreLocationUri, "ignoreLocationUri", None, "or")

        self._setQueryArrVal(ignoreLang, "ignoreLang", None, "or")

        self._setValIfNotDefault("keywordLoc", keywordsLoc, "body")
        self._setValIfNotDefault("ignoreKeywordLoc", ignoreKeywordsLoc, "body")

        self.addRequestedResult(requestedResult or RequestEventsInfo())


    def _getPath(self):
        return "/json/event"


    def addRequestedResult(self, requestEvents):
        """
        Add a result type that you would like to be returned.
        In case you are a subscribed customer you can ask for multiple result types in a single query (for free users, only a single result type can be required per call).
        Result types can be the classes that extend RequestEvents base class (see classes below).
        """
        assert isinstance(requestEvents, RequestEvents), "QueryEvents class can only accept result requests that are of type RequestEvents"
        self.resultTypeList = [item for item in self.resultTypeList if item.getResultType() != requestEvents.getResultType()]
        self.resultTypeList.append(requestEvents)


    def setRequestedResult(self, requestEvents):
        """
        Set the single result type that you would like to be returned. Any previously set result types will be overwritten.
        Result types can be the classes that extend RequestEvents base class (see classes below).
        """
        assert isinstance(requestEvents, RequestEvents), "QueryEvents class can only accept result requests that are of type RequestEvents"
        self.resultTypeList = [requestEvents]


    @staticmethod
    def initWithEventUriList(uriList):
        """
        Set a custom list of event uris. The results will be then computed on this list - no query will be done (all conditions will be ignored).
        """
        q = QueryEvents()
        assert isinstance(uriList, list), "uriList has to be a list of strings that represent event uris"
        q.queryParams = { "action": "getEvents", "eventUriList": ",".join(uriList) }
        return q


    @staticmethod
    def initWithComplexQuery(query):
        """
        create a query using a complex event query
        """
        q = QueryEvents()
        # provided an instance of ComplexEventQuery
        if isinstance(query, ComplexEventQuery):
            q._setVal("query", json.dumps(query.getQuery()))
        # provided query as a string containing the json object
        elif isinstance(query, six.string_types):
            foo = json.loads(query)
            q._setVal("query", query)
        # provided query as a python dict
        elif isinstance(query, dict):
            q._setVal("query", json.dumps(query))
        # unrecognized value provided
        else:
            assert False, "The instance of query parameter was not a ComplexEventQuery, a string or a python dict"
        return q



class QueryEventsIter(QueryEvents, six.Iterator):
    """
    class that simplifies and combines functionality from QueryEvents and RequestEventsInfo. It provides an iterator
    over the list of events that match the specified conditions
    """

    def count(self, eventRegistry):
        """
        return the number of events that match the criteria
        """
        self.setRequestedResult(RequestEventsUriList())
        res = eventRegistry.execQuery(self)
        if "error" in res:
            print(res["error"])
        count = res.get("uriList", {}).get("totalResults", 0)
        return count


    def execQuery(self, eventRegistry,
                  sortBy = "rel",
                  sortByAsc = False,
                  returnInfo = ReturnInfo(),
                  maxItems = -1,
                  **kwargs):
        """
        @param eventRegistry: instance of EventRegistry class. used to query new event list and uris
        @param sortBy: how should the resulting events be sorted. Options: date (by event date), rel (relevance to the query), size (number of articles),
            socialScore (amount of shares in social media), none (no specific sorting)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param maxItems: maximum number of items to be returned. Used to stop iteration sooner than results run out
        """
        self._er = eventRegistry
        self._sortBy = sortBy
        self._sortByAsc = sortByAsc
        self._returnInfo = returnInfo
        self._eventBatchSize = 50      # always download max - best for the user since it uses his token and we want to download as much as possible in a single search
        self._uriPage = 0
        self._useArchive = None
        # if we want to return only a subset of items:
        self._maxItems = maxItems
        self._currItem = 0
        # list of cached events that are yet to be returned by the iterator
        self._eventList = []
        self._uriWgtList = []
        # how many pages do we have for URIs. set once we call _getNextUriPage first
        self._allUriPages = None
        return self


    @staticmethod
    def initWithComplexQuery(query):
        q = QueryEventsIter()
        # provided an instance of ComplexEventQuery
        if isinstance(query, ComplexEventQuery):
            q._setVal("query", json.dumps(query.getQuery()))
        # provided query as a string containing the json object
        elif isinstance(query, six.string_types):
            foo = json.loads(query)
            q._setVal("query", query)
        # provided query as a python dict
        elif isinstance(query, dict):
            q._setVal("query", json.dumps(query))
        else:
            assert False, "The instance of query parameter was not a ComplexEventQuery, a string or a python dict"
        return q


    def _getNextUriPage(self):
        """download a simple list of event uris"""
        self._uriPage += 1
        self._uriWgtList = []
        if self._allUriPages != None and self._uriPage > self._allUriPages:
            return
        if self._er._verboseOutput:
            print("Downoading page %d of event uris" % (self._uriPage))
        self.setRequestedResult(RequestEventsUriWgtList(page = self._uriPage, sortBy = self._sortBy, sortByAsc = self._sortByAsc))
        res = self._er.execQuery(self)
        # remember if the archive needed to be used to process the query - use the info later when asking for the events in batches
        self._useArchive = self._er.getLastReqArchiveUse()
        if "error" in res:
            print(res["error"])
        self._uriWgtList = res.get("uriWgtList", {}).get("results", [])
        self._allUriPages = res.get("uriWgtList", {}).get("pages", 0)


    def _getNextEventBatch(self):
        """download next batch of events based on the event uris in the uri list"""
        self.clearRequestedResults()
        # try to get more uris, if none
        if len(self._uriWgtList) == 0:
            self._getNextUriPage()
        # if still no uris, then we have nothing to download
        if len(self._uriWgtList) == 0:
            return
        # get uris to download
        uriWgts = self._uriWgtList[:self._eventBatchSize]
        # create a list of uris, without the weights
        uriToWgts = dict([val.split(":") for val in uriWgts])
        uris = [val.split(":")[0] for val in uriWgts]
        # remove used uris
        self._uriWgtList = self._uriWgtList[self._eventBatchSize:]
        if self._er._verboseOutput:
            print("Downoading %d events..." % (len(uris)))
        q = QueryEvents.initWithEventUriList(uris)
        q.setRequestedResult(RequestEventsInfo(page = 1, count = self._eventBatchSize, sortBy = "none", returnInfo = self._returnInfo))
        # download articles and make sure that we set the same archive flag as it was returned when we were processing the uriList request
        res = self._er.execQuery(q, allowUseOfArchive = self._useArchive)
        if "error" in res:
            print("Error while obtaining a list of events: " + res["error"])
        else:
            assert res.get("events", {}).get("pages", 0) == 1
        results = res.get("events", {}).get("results", [])
        for result in results:
            if "uri" in result:
                result["wgt"] = int(uriToWgts.get(result["uri"], "1"))
        self._eventList.extend(results)


    def __iter__(self):
        return self


    def __next__(self):
        """iterate over the available events"""
        self._currItem += 1
        # if we want to return only the first X items, then finish once reached
        if self._maxItems >= 0 and self._currItem > self._maxItems:
            raise StopIteration
        if len(self._eventList) == 0:
            self._getNextEventBatch()
        if len(self._eventList) > 0:
            return self._eventList.pop(0)
        raise StopIteration



class RequestEvents:
    def __init__(self):
        self.resultType = None


    def getResultType(self):
        return self.resultType



class RequestEventsInfo(RequestEvents):
    def __init__(self, page = 1,
                 count = 20,
                 sortBy = "rel", sortByAsc = False,
                 returnInfo = ReturnInfo()):
        """
        return event details for resulting events
        @param page: page of the results to return (1, 2, ...)
        @param count: number of events to return per page (at most 50)
        @param sortBy: how should the resulting events be sorted. Options: date (by event date), rel (relevance to the query), size (number of articles),
            socialScore (amount of shares in social media), none (no specific sorting)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 50, "at most 50 events can be returned per call"
        self.resultType = "events"
        self.eventsPage = page
        self.eventsCount = count
        self.eventsSortBy = sortBy
        self.eventsSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("events"))


    def setPage(self, page):
        assert page >= 1, "page has to be >= 1"
        self.eventsPage = page


    def setCount(self, count):
        self.eventsCount = count



class RequestEventsUriList(RequestEvents):
    def __init__(self,
                 page = 1,
                 count = 100000,
                 sortBy = "rel", sortByAsc = False):
        """
        return a simple list of event uris for resulting events
        @param page: page of the results (1, 2, ...)
        @param count: number of results to include per page (at most 300000)
        @param sortBy: how should the resulting events be sorted. Options: date (by event date), rel (relevance to the query), size (number of articles),
            socialScore (amount of shares in social media), none (no specific sorting)
        @param sortByAsc: should the events be sorted in ascending order (True) or descending (False)
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 300000
        self.resultType = "uriList"
        self.uriListPage = page
        self.uriListCount = count
        self.uriListSortBy = sortBy
        self.uriListSortByAsc = sortByAsc

    def setPage(self, page):
        assert page >= 1, "page has to be >= 1"
        self.uriListPage = page



class RequestEventsUriWgtList(RequestEvents):
    def __init__(self,
                 page = 1,
                 count = 100000,
                 sortBy = "rel", sortByAsc = False):
        """
        return a simple list of event uris together with the scores for resulting events
        @param page: page of the results (1, 2, ...)
        @param count: number of results to include per page (at most 300000)
        @param sortBy: how should the resulting events be sorted. Options: date (by event date), rel (relevance to the query), size (number of articles),
            socialScore (amount of shares in social media), none (no specific sorting)
        @param sortByAsc: should the events be sorted in ascending order (True) or descending (False)
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 300000
        self.resultType = "uriWgtList"
        self.uriWgtListPage = page
        self.uriWgtListCount = count
        self.uriWgtListSortBy = sortBy
        self.uriWgtListSortByAsc = sortByAsc

    def setPage(self, page):
        assert page >= 1, "page has to be >= 1"
        self.uriWgtListPage = page



class RequestEventsTimeAggr(RequestEvents):
    def __init__(self):
        """
        return time distribution of resulting events
        """
        self.resultType = "timeAggr"



class RequestEventsKeywordAggr(RequestEvents):
    def __init__(self, lang = "eng"):
        """
        return keyword aggregate (tag cloud) on words in articles in resulting events
        @param lang: in which language to produce the list of top keywords
        """
        self.resultType = "keywordAggr"
        self.keywordAggrLang = lang



class RequestEventsLocAggr(RequestEvents):
    def __init__(self,
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        return aggreate of locations of resulting events
        @param eventsSampleSize: sample of events to use to compute the location aggregate (at most 300000)
        @param returnInfo: what details (about locations) should be included in the returned information
        """
        assert eventsSampleSize <= 300000
        self.resultType = "locAggr"
        self.locAggrSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("locAggr"))



class RequestEventsLocTimeAggr(RequestEvents):

    def __init__(self,
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        return aggreate of locations and times of resulting events
        @param eventsSampleSize: sample of events to use to compute the location aggregate (at most 300000)
        @param returnInfo: what details (about locations) should be included in the returned information
        """
        assert eventsSampleSize <= 300000
        self.resultType = "locTimeAggr"
        self.locTimeAggrSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("locTimeAggr"))



class RequestEventsConceptAggr(RequestEvents):
    def __init__(self,
                 conceptCount = 20,
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        compute which concept are the most frequently occuring in the list of resulting events
        @param conceptCount: number of top concepts to return (at most 200)
        @param eventsSampleSize: on what sample of results should the aggregate be computed (at most 3000000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 200
        assert eventsSampleSize <= 3000000
        self.resultType = "conceptAggr"
        self.conceptAggrConceptCount = conceptCount
        self.conceptAggrSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("conceptAggr"))



class RequestEventsConceptGraph(RequestEvents):
    def __init__(self,
                 conceptCount = 25,
                 linkCount = 50,
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        compute which concept pairs frequently co-occur together in the resulting events
        @param conceptCount: number of top concepts to return (at most 1000)
        @param linkCount: number of links between the concepts to return (at most 2000)
        @param eventsSampleSize: on what sample of results should the aggregate be computed (at most 300000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 1000
        assert linkCount <= 2000
        assert eventsSampleSize <= 300000
        self.resultType = "conceptGraph"
        self.conceptGraphConceptCount = conceptCount
        self.conceptGraphLinkCount = linkCount
        self.conceptGraphSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("conceptGraph"))



class RequestEventsConceptMatrix(RequestEvents):
    def __init__(self,
                 conceptCount = 25,
                 measure = "pmi",
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        get a matrix of concepts and their dependencies. For individual concept pairs
        return how frequently they co-occur in the resulting events and
        how "surprising" this is, based on the frequency of individual concepts
        @param conceptCount: number of top concepts to return (at most 200)
        @param measure: how should the interestingness between the selected pairs of concepts be computed. Options: pmi (pointwise mutual information), pairTfIdf (pair frequence * IDF of individual concepts), chiSquare
        @param eventsSampleSize: on what sample of results should the aggregate be computed (at most 300000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 200
        assert eventsSampleSize <= 300000
        self.resultType = "conceptMatrix"
        self.conceptMatrixConceptCount = conceptCount
        self.conceptMatrixMeasure = measure
        self.conceptMatrixSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("conceptMatrix"))



class RequestEventsConceptTrends(RequestEvents):
    def __init__(self,
                 conceptCount = 10,
                 returnInfo = ReturnInfo()):
        """
        return a list of top trending concepts and their daily trending info over time
        @param conceptCount: number of top concepts to return (at most 50)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 50
        self.resultType = "conceptTrends"
        self.conceptTrendsConceptCount = conceptCount
        self.__dict__.update(returnInfo.getParams("conceptTrends"))



class RequestEventsSourceAggr(RequestEvents):
    def __init__(self,
                 sourceCount = 30,
                 eventsSampleSize = 100000,
                 returnInfo = ReturnInfo()):
        """
        return top news sources that report about the events that match the search conditions
        @param sourceCount: number of top sources to return (at most 200)
        @param eventsSampleSize: on what sample of results should the aggregate be computed (at most 300000)
        @param returnInfo: what details about the sources should be included in the returned information
        """
        assert sourceCount <= 200
        assert eventsSampleSize <= 300000
        self.resultType = "sourceAggr"
        self.sourceAggrSourceCount = sourceCount
        self.sourceAggrSampleSize = eventsSampleSize
        self.__dict__.update(returnInfo.getParams("sourceAggr"))



class RequestEventsDateMentionAggr(RequestEvents):
    def __init__(self,
                 minDaysApart = 0,
                 minDateMentionCount = 5,
                 eventsSampleSize = 100000):
        """
        return events and the dates that are mentioned in articles about these events
        @param minDaysApart: ignore events that don't have a date that is more than this number of days apart from the tested event
        @param minDateMentionCount: report only dates that are mentioned at least this number of times
        @param eventsSampleSize: on what sample of results should the aggregate be computed (at most 300000)
        """
        assert eventsSampleSize <= 300000
        self.resultType = "dateMentionAggr"
        self.dateMentionAggrMinDaysApart = minDaysApart
        self.dateMentionAggrMinDateMentionCount = minDateMentionCount
        self.dateMentionAggrSampleSize = eventsSampleSize



class RequestEventsEventClusters(RequestEvents):
    def __init__(self,
                 keywordCount = 30,
                 maxEventsToCluster = 10000,
                 returnInfo = ReturnInfo()):
        """
        return hierarchical clustering of events into smaller clusters. 2-means clustering is applied on each node in the tree
        @param keywordCount: number of keywords to report in each of the clusters (at most !00)
        @param maxEventsToCluster: try to cluster at most this number of events (at most 10000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert keywordCount <= 100
        assert maxEventsToCluster <= 10000
        self.resultType = "eventClusters"
        self.eventClustersKeywordCount = keywordCount
        self.eventClustersMaxEventsToCluster = maxEventsToCluster
        self.__dict__.update(returnInfo.getParams("eventClusters"))



class RequestEventsCategoryAggr(RequestEvents):
    def __init__(self,
                 returnInfo = ReturnInfo()):
        """
        return distribution of events into dmoz categories
        @param returnInfo: what details about the categories should be included in the returned information
        """
        self.resultType = "categoryAggr"
        self.__dict__.update(returnInfo.getParams("categoryAggr"))



class RequestEventsRecentActivity(RequestEvents):
    def __init__(self,
                 maxEventCount = 60,
                 updatesAfterTm = None,
                 updatesAfterMinsAgo = None,
                 mandatoryLocation = True,
                 lang = None,
                 minAvgCosSim = 0,
                 returnInfo = ReturnInfo()):
        """
        return a list of recently changed events that match search conditions
        @param maxEventCount: max events to return (at most 500)
        @param updatesAfterTm: the time after which the events were added/updated (returned by previous call to the same method)
        @param updatesAfterMinsAgo: how many minutes into the past should we check (set either this or updatesAfterTm property, but not both)
        @param mandatoryLocation: return only events that have a geographic location assigned to them
        @param lang: limit the results to events that are described in the selected language (None if not filtered by any language)
        @param minAvgCosSim: the minimum avg cos sim of the events to be returned (events with lower quality should not be included)
        @param returnInfo: what details should be included in the returned information
        """
        assert maxEventCount <= 2000
        assert updatesAfterTm == None or updatesAfterMinsAgo == None, "You should specify either updatesAfterTm or updatesAfterMinsAgo parameter, but not both"
        self.resultType = "recentActivity"
        self.recentActivityEventsMaxEventCount = maxEventCount
        self.recentActivityEventsMandatoryLocation = mandatoryLocation
        if updatesAfterTm != None:
            self.recentActivityEventsUpdatesAfterTm = QueryParamsBase.encodeDateTime(updatesAfterTm)
        if updatesAfterMinsAgo != None:
            self.recentActivityEventsUpdatesAfterMinsAgo = updatesAfterMinsAgo
        if lang != None:
            self.recentActivityEventsLang = lang
        self.recentActivityEventsMinAvgCosSim = minAvgCosSim
        self.__dict__.update(returnInfo.getParams("recentActivityEvents"))





"""
Using the bottom classes you can obtain information about articles and events that
were shared the most on social media (Twitter and Facebook) on a particular day.
Given a date, articles published on that date are checked and top shared ones are returned. For an event,
events on that day are checked and top shared ones are returned.
Social score for an article is computed as the sum of shares on facebook and twitter.
Social score for an event is computed by checking 30 top shared articles in the event and averaging their social scores.
"""
from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class DailySharesBase(QueryParamsBase):
    def _getPath(self):
        return "/json/topDailyShares"

# get top shared articles for today or any other day
class GetTopSharedArticles(DailySharesBase):
    def __init__(self,
                 date = None,     # specify the date (either in YYYY-MM-DD or datetime.date format) for which to return top shared articles. If None then today is used
                 count = 20,      # number of top shared articles to return
                 returnInfo = ReturnInfo()):
        QueryParamsBase.__init__(self)
        self._setVal("action", "getArticles")
        self._setVal("count", count)
        self._update(returnInfo.getParams())

        if date == None:
            date = datetime.date.today()
        self._setDateVal("date", date)


# get top shared events for today or any other day
class GetTopSharedEvents(DailySharesBase):
    def __init__(self,
                 date = None,     # specify the date (either in YYYY-MM-DD or datetime.date format) for which to return top shared articles. If None then today is used
                 count = 20,      # number of top shared articles to return
                 returnInfo = ReturnInfo()):
        QueryParamsBase.__init__(self)
        self._setVal("action", "getEvents")
        self._setVal("count", count)
        self._update(returnInfo.getParams())

        if date == None:
            date = datetime.date.today()
        self._setDateVal("date", date)



import six, json
from eventregistry.Base import *
from eventregistry.ReturnInfo import *
from eventregistry.Query import *


class QueryArticles(Query):
    def __init__(self,
                 keywords = None,
                 conceptUri = None,
                 categoryUri = None,
                 sourceUri = None,
                 sourceLocationUri = None,
                 sourceGroupUri = None,
                 locationUri = None,
                 lang = None,
                 dateStart = None,
                 dateEnd = None,
                 dateMentionStart = None,
                 dateMentionEnd = None,
                 ignoreKeywords = None,
                 ignoreConceptUri = None,
                 ignoreCategoryUri = None,
                 ignoreSourceUri = None,
                 ignoreSourceLocationUri = None,
                 ignoreSourceGroupUri = None,
                 ignoreLocationUri = None,
                 ignoreLang = None,
                 keywordsLoc = "body",
                 ignoreKeywordsLoc = "body",
                 isDuplicateFilter = "keepAll",
                 hasDuplicateFilter = "keepAll",
                 eventFilter = "keepAll",
                 requestedResult = None):
        """
        Query class for searching for individual articles in the Event Registry.
        The resulting articles have to match all specified conditions. If a parameter value equals "" or [], then it is ignored.
        In order for query to be valid, it has to have at least one positive condition (condition that does not start with ignore*).

        @param keywords: find articles that mention the specified keywords.
            A single keyword/phrase can be provided as a string, multiple keywords/phrases can be provided as a list of strings.
            Use QueryItems.AND() if *all* provided keywords/phrases should be mentioned, or QueryItems.OR() if *any* of the keywords/phrases should be mentioned.
            or QueryItems.OR() to specify a list of keywords where any of the keywords have to appear
        @param conceptUri: find articles where the concept with concept uri is mentioned.
            A single concept uri can be provided as a string, multiple concept uris can be provided as a list of strings.
            Use QueryItems.AND() if *all* provided concepts should be mentioned, or QueryItems.OR() if *any* of the concepts should be mentioned.
            To obtain a concept uri using a concept label use EventRegistry.getConceptUri().
        @param categoryUri: find articles that are assigned into a particular category.
            A single category can be provided as a string, while multiple categories can be provided as a list in QueryItems.AND() or QueryItems.OR().
            A category uri can be obtained from a category name using EventRegistry.getCategoryUri().
        @param sourceUri: find articles that were written by a news source sourceUri.
            If multiple sources should be considered use QueryItems.OR() to provide the list of sources.
            Source uri for a given news source name can be obtained using EventRegistry.getNewsSourceUri().
        @param sourceLocationUri: find articles that were written by news sources located in the given geographic location.
            If multiple source locations are provided, then put them into a list inside QueryItems.OR()
            Location uri can either be a city or a country. Location uri for a given name can be obtained using EventRegistry.getLocationUri().
        @param sourceGroupUri: find articles that were written by news sources that are assigned to the specified source group.
            If multiple source groups are provided, then put them into a list inside QueryItems.OR()
            Source group uri for a given name can be obtained using EventRegistry.getSourceGroupUri().
        @param locationUri: find articles that describe something that occured at a particular location.
            If value can be a string or a list of strings provided in QueryItems.OR().
            Location uri can either be a city or a country. Location uri for a given name can be obtained using EventRegistry.getLocationUri().
        @param lang: find articles that are written in the specified language.
            If more than one language is specified, resulting articles has to be written in *any* of the languages.
        @param dateStart: find articles that were written on or after dateStart. Date should be provided in YYYY-MM-DD format, datetime.time or datetime.datetime.
        @param dateEnd: find articles that occured before or on dateEnd. Date should be provided in YYYY-MM-DD format, datetime.time or datetime.datetime.
        @param dateMentionStart: find articles that explicitly mention a date that is equal or greater than dateMentionStart.
        @param dateMentionEnd: find articles that explicitly mention a date that is lower or equal to dateMentionEnd.
        @param ignoreKeywords: ignore articles that mention all provided keywords
        @param ignoreConceptUri: ignore articles that mention all provided concepts
        @param ignoreCategoryUri: ignore articles that are assigned to a particular category
        @param ignoreSourceUri: ignore articles that have been written by *any* of the specified news sources
        @param ignoreSourceLocationUri: ignore articles that have been written by sources located at *any* of the specified locations
        @param ignoreSourceGroupUri: ignore articles that have been written by sources in *any* of the specified source groups
        @param ignoreLocationUri: ignore articles that occured in any of the provided locations. A location can be a city or a place
        @param ignoreLang: ignore articles that are written in *any* of the provided languages
        @param keywordsLoc: where should we look when searching using the keywords provided by "keywords" parameter. "body" (default), "title", or "body,title"
        @param ignoreKeywordsLoc: where should we look when data should be used when searching using the keywords provided by "ignoreKeywords" parameter. "body" (default), "title", or "body,title"
        @param isDuplicateFilter: some articles can be duplicates of other articles. What should be done with them. Possible values are:
                "skipDuplicates" (skip the resulting articles that are duplicates of other articles)
                "keepOnlyDuplicates" (return only the duplicate articles)
                "keepAll" (no filtering, default)
        @param hasDuplicateFilter: some articles are later copied by others. What should be done with such articles. Possible values are:
                "skipHasDuplicates" (skip the resulting articles that have been later copied by others)
                "keepOnlyHasDuplicates" (return only the articles that have been later copied by others)
                "keepAll" (no filtering, default)
        @param eventFilter: some articles describe a known event and some don't. This filter allows you to filter the resulting articles based on this criteria.
                Possible values are:
                "skipArticlesWithoutEvent" (skip articles that are not describing any known event in ER)
                "keepOnlyArticlesWithoutEvent" (return only the articles that are not describing any known event in ER)
                "keepAll" (no filtering, default)
        @param requestedResult: the information to return as the result of the query. By default return the list of matching articles
        """
        super(QueryArticles, self).__init__()
        self._setVal("action", "getArticles")

        self._setQueryArrVal(keywords, "keyword", "keywordOper", "and")
        self._setQueryArrVal(conceptUri, "conceptUri", "conceptOper", "and")
        self._setQueryArrVal(categoryUri, "categoryUri", "categoryOper", "or")
        self._setQueryArrVal(sourceUri, "sourceUri", None, "or")
        self._setQueryArrVal(sourceLocationUri, "sourceLocationUri", None, "or")
        self._setQueryArrVal(sourceGroupUri, "sourceGroupUri", None, "or")
        self._setQueryArrVal(locationUri, "locationUri", None, "or")        # location such as "http://en.wikipedia.org/wiki/Ljubljana"

        self._setQueryArrVal(lang, "lang", None, "or")                      # a single lang or list (possible: eng, deu, spa, zho, slv)

        # starting date of the published articles (e.g. 2014-05-02)
        if dateStart != None:
            self._setDateVal("dateStart", dateStart)
        # ending date of the published articles (e.g. 2014-05-02)
        if dateEnd != None:
            self._setDateVal("dateEnd", dateEnd)

        # first valid mentioned date detected in articles (e.g. 2014-05-02)
        if dateMentionStart != None:
            self._setDateVal("dateMentionStart", dateMentionStart)
        # last valid mentioned date detected in articles (e.g. 2014-05-02)
        if dateMentionEnd != None:
            self._setDateVal("dateMentionEnd", dateMentionEnd)

        # for the negative conditions, only the OR is a valid operator type
        self._setQueryArrVal(ignoreKeywords, "ignoreKeyword", None, "or")
        self._setQueryArrVal(ignoreConceptUri, "ignoreConceptUri", None, "or")
        self._setQueryArrVal(ignoreCategoryUri, "ignoreCategoryUri", None, "or")
        self._setQueryArrVal(ignoreSourceUri, "ignoreSourceUri", None, "or")
        self._setQueryArrVal(ignoreSourceLocationUri, "ignoreSourceLocationUri", None, "or")
        self._setQueryArrVal(ignoreSourceGroupUri, "ignoreSourceGroupUri", None, "or")
        self._setQueryArrVal(ignoreLocationUri, "ignoreLocationUri", None, "or")

        self._setQueryArrVal(ignoreLang, "ignoreLang", None, "or")

        self._setValIfNotDefault("keywordLoc", keywordsLoc, "body")
        self._setValIfNotDefault("ignoreKeywordLoc", ignoreKeywordsLoc, "body")

        self._setValIfNotDefault("isDuplicateFilter", isDuplicateFilter, "keepAll")
        self._setValIfNotDefault("hasDuplicateFilter", hasDuplicateFilter, "keepAll")
        self._setValIfNotDefault("eventFilter", eventFilter, "keepAll")

        # set the information that should be returned
        self.setRequestedResult(requestedResult or RequestArticlesInfo())


    def _getPath(self):
        return "/json/article"


    def addRequestedResult(self, requestArticles):
        """
        Add a result type that you would like to be returned.
        In case you are a subscribed customer you can ask for multiple result types in a single query (for free users, only a single result type can be required per call).
        Result types can be the classes that extend RequestArticles base class (see classes below).
        """
        assert isinstance(requestArticles, RequestArticles), "QueryArticles class can only accept result requests that are of type RequestArticles"
        self.resultTypeList = [item for item in self.resultTypeList if item.getResultType() != requestArticles.getResultType()]
        self.resultTypeList.append(requestArticles)


    def setRequestedResult(self, requestArticles):
        """
        Set the single result type that you would like to be returned. Any previously set result types will be overwritten.
        Result types can be the classes that extend RequestArticles base class (see classes below).
        """
        assert isinstance(requestArticles, RequestArticles), "QueryArticles class can only accept result requests that are of type RequestArticles"
        self.resultTypeList = [requestArticles]


    @staticmethod
    def initWithArticleUriList(uriList):
        """
        instead of making a query, provide a list of article URIs manually, and then produce the desired results on top of them
        """
        q = QueryArticles()
        assert isinstance(uriList, list), "uriList has to be a list of strings that represent article uris"
        q.queryParams = { "action": "getArticles", "articleUri": uriList }
        return q


    @staticmethod
    def initWithComplexQuery(query):
        """
        create a query using a complex article query
        """
        q = QueryArticles()
        # provided an instance of ComplexArticleQuery
        if isinstance(query, ComplexArticleQuery):
            q._setVal("query", json.dumps(query.getQuery()))
        # provided query as a string containing the json object
        elif isinstance(query, six.string_types):
            foo = json.loads(query)
            q._setVal("query", query)
        # provided query as a python dict
        elif isinstance(query, dict):
            q._setVal("query", json.dumps(query))
        else:
            assert False, "The instance of query parameter was not a ComplexArticleQuery, a string or a python dict"
        return q



class QueryArticlesIter(QueryArticles, six.Iterator):
    """
    class that simplifies and combines functionality from QueryArticles and RequestArticlesInfo. It provides an iterator
    over the list of articles that match the specified conditions
    """

    def count(self, eventRegistry):
        """
        return the number of articles that match the criteria
        """
        self.setRequestedResult(RequestArticlesUriList())
        res = eventRegistry.execQuery(self)
        if "error" in res:
            print(res["error"])
        count = res.get("uriList", {}).get("totalResults", 0)
        return count


    def execQuery(self, eventRegistry,
                  sortBy = "rel",
                  sortByAsc = False,
                  returnInfo = ReturnInfo(),
                  articleBatchSize = 100,
                  maxItems = -1,
                  **kwargs):
        """
        @param eventRegistry: instance of EventRegistry class. used to query new article list and uris
        @param sortBy: how are articles sorted. Options: id (internal id), date (publishing date), cosSim (closeness to the event centroid), rel (relevance to the query), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param returnInfo: what details should be included in the returned information
        @param maxItems: maximum number of items to be returned. Used to stop iteration sooner than results run out
        """
        self._er = eventRegistry
        self._sortBy = sortBy
        self._sortByAsc = sortByAsc
        self._returnInfo = returnInfo
        self._articleBatchSize = 100    # always download 100 - best for the user since it uses his token and we want to download as much as possible in a single search
        self._uriPage = 0
        self._useArchive = None
        # if we want to return only a subset of items:
        self._maxItems = maxItems
        self._currItem = 0
        # list of cached articles that are yet to be returned by the iterator
        self._articleList = []
        self._uriWgtList = []
        # how many pages do we have for URIs. set once we call _getNextUriPage first
        self._allUriPages = None
        return self


    @staticmethod
    def initWithComplexQuery(query):
        q = QueryArticlesIter()
        # provided an instance of ComplexArticleQuery
        if isinstance(query, ComplexArticleQuery):
            q._setVal("query", json.dumps(query.getQuery()))
        # provided query as a string containing the json object
        elif isinstance(query, six.string_types):
            foo = json.loads(query)
            q._setVal("query", query)
        # provided query as a python dict
        elif isinstance(query, dict):
            q._setVal("query", json.dumps(query))
        else:
            assert False, "The instance of query parameter was not a ComplexArticleQuery, a string or a python dict"
        return q


    def _getNextUriPage(self):
        """download a simple list of article uris"""
        self._uriPage += 1
        self._uriWgtList = []
        if self._allUriPages != None and self._uriPage > self._allUriPages:
            return
        if self._er._verboseOutput:
            print("Downoading page %d of article uris" % (self._uriPage))
        self.setRequestedResult(RequestArticlesUriWgtList(page = self._uriPage, sortBy = self._sortBy, sortByAsc = self._sortByAsc))
        res = self._er.execQuery(self)
        # remember if the archive needed to be used to process the query - use the info later when asking for the articles in batches
        self._useArchive = self._er.getLastReqArchiveUse()
        if "error" in res:
            print(res["error"])
        self._uriWgtList = res.get("uriWgtList", {}).get("results", [])
        self._allUriPages = res.get("uriWgtList", {}).get("pages", 0)


    def _getNextArticleBatch(self):
        """download next batch of articles based on the article uris in the uri list"""
        # try to get more uris, if none
        if len(self._uriWgtList) == 0:
            self._getNextUriPage()
        # if still no uris, then we have nothing to download
        if len(self._uriWgtList) == 0:
            return
        # get uris to download
        uriWgts = self._uriWgtList[:self._articleBatchSize]
        # create a list of uris, without the weights
        uriToWgts = dict([val.split(":") for val in uriWgts])
        uris = [val.split(":")[0] for val in uriWgts]
        # remove used uris
        self._uriWgtList = self._uriWgtList[self._articleBatchSize:]
        if self._er._verboseOutput:
            print("Downoading %d articles..." % (len(uris)))

        q = QueryArticles.initWithArticleUriList(uris)
        q.setRequestedResult(RequestArticlesInfo(page = 1, count = self._articleBatchSize, sortBy = "none", returnInfo = self._returnInfo))
        # download articles and make sure that we set the same archive flag as it was returned when we were processing the uriList request
        res = self._er.execQuery(q, allowUseOfArchive = self._useArchive)
        if "error" in res:
            print("Error while obtaining a list of articles: " + res["error"])
        else:
            assert res.get("articles", {}).get("pages", 0) == 1
        results = res.get("articles", {}).get("results", [])
        for result in results:
            if "uri" in result:
                result["wgt"] = int(uriToWgts.get(result["uri"], "1"))
        self._articleList.extend(results)


    def __iter__(self):
        return self


    def __next__(self):
        """iterate over the available articles"""
        self._currItem += 1
        # if we want to return only the first X items, then finish once reached
        if self._maxItems >= 0 and self._currItem > self._maxItems:
            raise StopIteration
        if len(self._articleList) == 0:
            self._getNextArticleBatch()
        if len(self._articleList) > 0:
            return self._articleList.pop(0)
        raise StopIteration



class RequestArticles:
    def __init__(self):
        self.resultType = None


    def getResultType(self):
        return self.resultType



class RequestArticlesInfo(RequestArticles):
    def __init__(self,
                 page = 1,
                 count = 20,
                 sortBy = "date", sortByAsc = False,
                 returnInfo = ReturnInfo()):
        """
        return article details for resulting articles
        @param page: page of the articles to return
        @param count: number of articles to return for the given page (at most 100)
        @param sortBy: how are articles sorted. Options: id (internal id), date (publishing date), cosSim (closeness to the event centroid), rel (relevance to the query), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 100, "at most 100 articles can be returned per call"
        self.resultType = "articles"
        self.articlesPage = page
        self.articlesCount = count
        self.articlesSortBy = sortBy
        self.articlesSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("articles"))


    def setPage(self, page):
        """
        set the page of results to obtain
        """
        assert page >= 1, "page has to be >= 1"
        self.articlesPage = page



class RequestArticlesUriList(RequestArticles):
    def __init__(self,
                 page = 1,
                 count = 10000,
                 sortBy = "fq", sortByAsc = False):
        """
        return a list of article uris
        @param page: page of the results (1, 2, ...)
        @param count: number of items to return in a single query (at most 50000)
        @param sortBy: how are articles sorted. Options: id (internal id), date (publishing date), cosSim (closeness to the event centroid), rel (relevance to the query), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False) according to the sortBy criteria
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 50000
        self.resultType = "uriList"
        self.uriListPage = page
        self.uriListCount = count
        self.uriListSortBy = sortBy
        self.uriListSortByAsc = sortByAsc


    def setPage(self, page):
        assert page >= 1, "page has to be >= 1"
        self.uriListPage = page



class RequestArticlesUriWgtList(RequestArticles):
    def __init__(self,
                 page = 1,
                 count = 10000,
                 sortBy = "fq", sortByAsc = False):
        """
        return a list of article uris together with the scores
        @param page: page of the results (1, 2, ...)
        @param count: number of items to return in a single query (at most 50000)
        @param sortBy: how are articles sorted. Options: id (internal id), date (publishing date), cosSim (closeness to the event centroid), rel (relevance to the query), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False) according to the sortBy criteria
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 50000
        self.resultType = "uriWgtList"
        self.uriWgtListPage = page
        self.uriWgtListCount = count
        self.uriWgtListSortBy = sortBy
        self.uriWgtListSortByAsc = sortByAsc


    def setPage(self, page):
        assert page >= 1, "page has to be >= 1"
        self.uriWgtListPage = page



class RequestArticlesUrlList(RequestArticles):
    def __init__(self,
                 page = 1,
                 count = 10000,
                 sortBy = "fq", sortByAsc = False):
        """
        return a list of article urls
        @param page: page of the results (1, 2, ...)
        @param count: number of items to return in a single query (at most 50000)
        @param sortBy: how are articles sorted. Options: id (internal id), date (publishing date), cosSim (closeness to the event centroid), rel (relevance to the query), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False) according to the sortBy criteria
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 50000
        self.resultType = "urlList"
        self.urlListPage = page
        self.urlListCount = count
        self.urlListSortBy = sortBy
        self.urlListSortByAsc = sortByAsc



class RequestArticlesTimeAggr(RequestArticles):
    def __init__(self):
        """
        return time distribution of resulting articles
        """
        self.resultType = "timeAggr"



class RequestArticlesConceptAggr(RequestArticles):
    def __init__(self,
                 conceptCount = 25,
                 articlesSampleSize = 10000,
                 returnInfo = ReturnInfo()):
        """
        get aggreate of concepts of resulting articles
        @param conceptCount: number of top concepts to return (at most 500)
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 20000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 500
        assert articlesSampleSize <= 20000
        self.resultType = "conceptAggr"
        self.conceptAggrConceptCount = conceptCount
        self.conceptAggrSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("conceptAggr"))



class RequestArticlesCategoryAggr(RequestArticles):
    def __init__(self,
                 articlesSampleSize = 20000,
                 returnInfo = ReturnInfo()):
        """
        return aggreate of categories of resulting articles
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 50000)
        @param returnInfo: what details about the categories should be included in the returned information
        """
        assert articlesSampleSize <= 50000
        self.resultType = "categoryAggr"
        self.categoryAggrSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("categoryAggr"))



class RequestArticlesSourceAggr(RequestArticles):
    def __init__(self,
                 articlesSampleSize = 20000,
                 returnInfo = ReturnInfo()):
        """
        get aggreate of news sources of resulting articles
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 1000000)
        @param returnInfo: what details about the sources should be included in the returned information
        """
        assert articlesSampleSize <= 1000000
        self.resultType = "sourceAggr"
        self.sourceAggrSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("sourceAggr"))


class RequestArticlesKeywordAggr(RequestArticles):
    def __init__(self,
                 lang = "eng",
                 articlesSampleSize = 10000):
        """
        get top keywords in the resulting articles
        @param lang: articles in which language should be analyzed and processed
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 50000)
        """
        assert articlesSampleSize <= 50000
        self.resultType = "keywordAggr"
        self.keywordAggrLang = lang
        self.keywordAggrSampleSize = articlesSampleSize



class RequestArticlesConceptGraph(RequestArticles):
    def __init__(self,
                 conceptCount = 25,
                 linkCount = 50,
                 articlesSampleSize = 10000,
                 returnInfo = ReturnInfo()):
        """
        get concept graph of resulting articles. Identify concepts that frequently co-occur with other concepts
        @param conceptCount: how many concepts should be returned (at most 1000)
        @param linkCount: how many top links between the concepts should be returned (at most 2000)
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 50000)
        @param returnInfo: what details about the concepts should be included in the returned information
        """
        assert conceptCount <= 1000
        assert linkCount <= 2000
        assert articlesSampleSize <= 50000
        self.resultType = "conceptGraph"
        self.conceptGraphConceptCount = conceptCount
        self.conceptGraphLinkCount = linkCount
        self.conceptGraphSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("conceptGraph"))



class RequestArticlesConceptMatrix(RequestArticles):
    def __init__(self,
                 conceptCount = 25,
                 measure = "pmi",
                 articlesSampleSize = 10000,
                 returnInfo = ReturnInfo()):
        """
        get aggreate of concept co-occurences of resulting articles
        @param conceptCount: how many concepts should be returned (at most 200)
        @param measure: how should the interestingness between the selected pairs of concepts be computed. Options: pmi (pointwise mutual information), pairTfIdf (pair frequence * IDF of individual concepts), chiSquare
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 50000)
        @param returnInfo: what details should be included in the returned information
        """
        assert conceptCount <= 200
        assert articlesSampleSize <= 50000
        self.resultType = "conceptMatrix"
        self.conceptMatrixConceptCount = conceptCount
        self.conceptMatrixMeasure = measure
        self.conceptMatrixSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("conceptMatrix"))



class RequestArticlesConceptTrends(RequestArticles):
    def __init__(self,
                 count = 25,
                 articlesSampleSize = 10000,
                 returnInfo = ReturnInfo()):
        """
        get trending of concepts in the resulting articles
        @param count: number of concepts to return (at most 50)
        @param articlesSampleSize: on what sample of results should the aggregate be computed (at most 50000)
        @param returnInfo: what details should be included in the returned information
        """
        assert count <= 50
        assert articlesSampleSize <= 50000
        self.resultType = "conceptTrends"
        self.conceptTrendsConceptCount = count
        self.conceptTrendsSampleSize = articlesSampleSize
        self.__dict__.update(returnInfo.getParams("conceptTrends"))



class RequestArticlesDateMentionAggr(RequestArticles):
    """
    get mentioned dates in the articles
    """
    def __init__(self):
        self.resultType = "dateMentionAggr"



class RequestArticlesRecentActivity(RequestArticles):
    def __init__(self,
                 maxArticleCount = 60,
                 updatesAfterTm = None,
                 updatesAfterMinsAgo = None,
                 lang = None,
                 mandatorySourceLocation = False,
                 returnInfo = ReturnInfo()):
        """
        get the list of articles that were recently added to the Event Registry and match the selected criteria
        @param maxArticleCount: max articles to return (at most 500)
        @param updatesAfterTm: the time after which the articles were added (returned by previous call to the same method)
        @param updatesAfterMinsAgo: how many minutes into the past should we check (set either this or updatesAfterTm property, but not both)
        @param lang: return only articles in the specified languages (None if no limits). accepts string or a list of strings
        @param mandatorySourceLocation: return only articles for which we know the source's geographic location
        @param returnInfo: what details should be included in the returned information
        """
        assert maxArticleCount <= 1000
        assert updatesAfterTm == None or updatesAfterMinsAgo == None, "You should specify either updatesAfterTm or updatesAfterMinsAgo parameter, but not both"
        self.resultType = "recentActivity"
        self.recentActivityArticlesMaxArticleCount  = maxArticleCount
        if updatesAfterTm != None:
            self.recentActivityArticlesUpdatesAfterTm = QueryParamsBase.encodeDateTime(updatesAfterTm)
        if updatesAfterMinsAgo != None:
            self.recentActivityEventsUpdatesAfterMinsAgo = updatesAfterMinsAgo
        if lang != None:
            self.recentActivityArticlesLang = lang
        self.recentActivityArticlesMandatorySourceLocation = mandatorySourceLocation
        self.__dict__.update(returnInfo.getParams("recentActivityArticles"))


"""
provides classes for getting new/updated events and articles
"""

from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class GetRecentEvents(QueryParamsBase):
    def __init__(self,
                 eventRegistry,
                 mandatoryLang = None,
                 mandatoryLocation = True,
                 returnInfo = ReturnInfo(),
                 **kwargs):
        """
        return info about recently modified events

        @param eventRegistry: instance of class EventRegistry
        @param mandatoryLang: set a lang or array of langs if you wish to only get events covered at least by the specified language
        @param mandatoryLocation: if set to True then return only events that have a known geographic location
        """
        QueryParamsBase.__init__(self)

        self._er = eventRegistry
        self._setVal("recentActivityEventsMandatoryLocation", mandatoryLocation)
        # return only events that have at least a story in the specified language
        if mandatoryLang != None:
            self._setVal("recentActivityEventsMandatoryLang", mandatoryLang)
        self.queryParams.update(kwargs)
        self._update(returnInfo.getParams("recentActivityEvents"))


    def _getPath(self):
        return "/json/minuteStreamEvents"


    def getUpdates(self):
        """
        Get the latest new or updated events from Event Registry
        NOTE: call this method exactly once per minute - calling it more frequently will return the same results multiple times,
        calling it less frequently will miss on some results. Results are computed once a minute.
        """
        # execute the query
        ret = self._er.execQuery(self)

        if ret and "recentActivity" in ret and "events" in ret["recentActivity"]:
            # return the updated information
            return ret["recentActivity"]["events"]
        # or empty
        return {}



class GetRecentArticles(QueryParamsBase):
    def __init__(self,
                 eventRegistry,
                 mandatorySourceLocation = False,
                 articleLang = None,
                 returnInfo = ReturnInfo(),
                 **kwargs):
        """
        return info about recently added articles

        @param eventRegistry: instance of class EventRegistry
        @param mandatorySourceLocation: if True then return only articles from sources for which we know geographic location
        @param articleLang: None, string or a list of strings, depending if we should return all articles, or articles in one or more languages
        """
        QueryParamsBase.__init__(self)

        self._er = eventRegistry
        self._setVal("recentActivityArticlesMandatorySourceLocation", mandatorySourceLocation)
        if articleLang != None:
            self._setVal("recentActivityArticlesLang", articleLang)
        self.queryParams.update(kwargs)
        self._update(returnInfo.getParams("recentActivityArticles"))


    def _getPath(self):
        return "/json/minuteStreamArticles"


    def getUpdates(self):
        """
        Get the latest new or updated events articles Event Registry.
        NOTE: call this method exactly once per minute - calling it more frequently will return the same results multiple times,
        calling it less frequently will miss on some results. Results are computed once a minute.
        """
        # execute the query
        ret = self._er.execQuery(self)

        if ret and "recentActivity" in ret and "articles" in ret["recentActivity"]:
            # return the latest articles
            return ret["recentActivity"]["articles"]["activity"]
        # or empty
        return []



import six
from eventregistry.Base import *
from eventregistry.ReturnInfo import *
from eventregistry.QueryArticle import QueryArticle, RequestArticleInfo


class QueryEvent(Query):
    """
    Class for obtaining available info for one or more events in the Event Registry
    """
    def __init__(self,
                 eventUriOrList,
                 requestedResult = None):
        """
        @param eventUriOrUriList: a single event uri or a list of event uris
        @param requestedResult: the information to return as the result of the query. By default return the details of the event
        """
        super(QueryEvent, self).__init__()
        self._setVal("action", "getEvent")
        self._setVal("eventUri", eventUriOrList)
        self.setRequestedResult(requestedResult or RequestEventInfo())


    def _getPath(self):
        return "/json/event"


    def addRequestedResult(self, requestEvent):
        """
        Add a result type that you would like to be returned.
        In case you are a subscribed customer you can ask for multiple result types in a single query (for free users, only a single result type can be required per call).
        Result types can be the classes that extend RequestEvent base class (see classes below).
        """
        assert isinstance(requestEvent, RequestEvent), "QueryEvent class can only accept result requests that are of type RequestEvent"
        self.resultTypeList = [item for item in self.resultTypeList if item.getResultType() != requestEvent.getResultType()]
        self.resultTypeList.append(requestEvent)


    def setRequestedResult(self, requestEvent):
        """
        Set the single result type that you would like to be returned. Any previously set result types will be overwritten.
        Result types can be the classes that extend RequestEvent base class (see classes below).
        """
        assert isinstance(requestEvent, RequestEvent), "QueryEvent class can only accept result requests that are of type RequestEvent"
        self.resultTypeList = [requestEvent]



class QueryEventArticlesIter(QueryEvent, six.Iterator):
    """
    Class for obtaining an iterator over all articles in the event
    """
    def __init__(self, eventUri):
        """@param eventUri: a single event for which we want to obtain the list of articles in it"""
        super(QueryEventArticlesIter, self).__init__(eventUri)


    def count(self, eventRegistry,
              lang = mainLangs):
        """
        return the number of articles that match the criteria
        @param eventRegistry: instance of EventRegistry class. used to obtain the necessary data
        @param lang: array or a single language in which to return the list of matching articles
        """
        self.setRequestedResult(RequestEventArticleUris(lang = lang))
        res = eventRegistry.execQuery(self)
        if "error" in res:
            print(res["error"])
        count = len(res.get(self.queryParams["eventUri"], {}).get("articleUris", {}).get("results", []))
        return count


    def execQuery(self, eventRegistry,
            lang = mainLangs,
            sortBy = "cosSim", sortByAsc = False,
            returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 200)),
            articleBatchSize = 200,
            maxItems = -1):
        """
        @param eventRegistry: instance of EventRegistry class. used to obtain the necessary data
        @param lang: array or a single language in which to return the list of matching articles
        @param sortBy: order in which event articles are sorted. Options: none (no specific sorting), id (internal id), date (published date), cosSim (closeness to event centroid), sourceImportance (manually curated score of source importance - high value, high importance), sourceImportanceRank (reverse of sourceImportance), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares on social media), facebookShares (shares on Facebook only)
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param returnInfo: what details should be included in the returned information
        @param articleBatchSize: number of articles to download at once (we are not downloading article by article) (at most 200)
        @param maxItems: maximum number of items to be returned. Used to stop iteration sooner than results run out
        """
        assert articleBatchSize <= 200, "You can not have a batch size > 200 items"
        self._er = eventRegistry
        self._lang = lang
        self._sortBy = sortBy
        self._sortByAsc = sortByAsc
        self._returnInfo = returnInfo
        self._articleBatchSize = articleBatchSize
        # if we want to return only a subset of items:
        self._maxItems = maxItems
        self._currItem = 0
        # download the list of article uris
        self._articleList = []
        self.setRequestedResult(RequestEventArticleUris(lang = self._lang, sortBy = self._sortBy, sortByAsc = self._sortByAsc))
        res = self._er.execQuery(self)
        if "error" in res:
            print(res["error"])
        self._uriList = res.get(self.queryParams["eventUri"], {}).get("articleUris", {}).get("results", [])
        return self


    def _getNextArticleBatch(self):
        """download next batch of events based on the event uris in the uri list"""
        self.clearRequestedResults()
        # if no uris, then we have nothing to download
        if len(self._uriList) == 0:
            return
        # get uris to download
        uris = self._uriList[:self._articleBatchSize]
        if self._er._verboseOutput:
            print("Downoading %d articles from event %s" % (len(uris), self.queryParams["eventUri"]))
        # remove used uris
        self._uriList = self._uriList[self._articleBatchSize:]
        q = QueryArticle(uris)
        q.setRequestedResult(RequestArticleInfo(self._returnInfo))
        res = self._er.execQuery(q)
        if "error" in res:
            print(res["error"])
        arts = [ res[key]["info"] for key in uris if key in res and "info" in res[key]]
        self._articleList.extend(arts)


    def __iter__(self):
        # clear any past info - iterator should start from beginning
        return self


    def __next__(self):
        """iterate over the available events"""
        self._currItem += 1
        # if we want to return only the first X items, then finish once reached
        if self._maxItems >= 0 and self._currItem > self._maxItems:
            raise StopIteration
        if len(self._articleList) == 0:
            self._getNextArticleBatch()
        if len(self._articleList) > 0:
            return self._articleList.pop(0)
        raise StopIteration



class RequestEvent:
    def __init__(self):
        self.resultType = None


    def getResultType(self):
        return self.resultType



class RequestEventInfo(RequestEvent):
    def __init__(self, returnInfo = ReturnInfo()):
        """
        return details about an event
        """
        self.resultType = "info"
        self.__dict__.update(returnInfo.getParams("info"))



class RequestEventArticles(RequestEvent):
    def __init__(self,
                 page = 1,
                 count = 20,
                 lang = mainLangs,
                 sortBy = "cosSim", sortByAsc = False,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 200))):
        """
        return articles about the event
        @param page: page of the articles to return (1, 2, ...)
        @param count: number of articles to return per page (at most 200)
        @param lang: a single lanugage or a list of languages in which to return the articles
        @param sortBy: order in which event articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to event centroid), sourceImportanceRank (importance of the news source, custom set), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares in social media)
        @param sortByAsc: should the articles be sorted in ascending order (True) or descending (False) based on sortBy value
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 200, "at most 200 articles can be returned per call"
        self.resultType = "articles"
        self.articlesPage = page
        self.articlesCount = count
        self.articlesLang = lang
        self.articlesSortBy = sortBy
        self.articlesSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("articles"))



class RequestEventArticleUris(RequestEvent):
    def __init__(self,
                 lang = mainLangs,
                 sortBy = "cosSim", sortByAsc = False  # order in which event articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to event centroid), socialScore (total shares in social media)
                 ):
        """
        return just a list of article uris
        @param lang: a single lanugage or a list of languages in which to return the articles
        @param sortBy: order in which event articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to event centroid), sourceImportanceRank (importance of the news source, custom set), sourceAlexaGlobalRank (global rank of the news source), sourceAlexaCountryRank (country rank of the news source), socialScore (total shares in social media)
        @param sortByAsc: should the articles be sorted in ascending order (True) or descending (False) based on sortBy value
        """
        self.articleUrisLang = lang
        self.articleUrisSortBy = sortBy
        self.articleUrisSortByAsc = sortByAsc
        self.resultType = "articleUris"



class RequestEventKeywordAggr(RequestEvent):
    def __init__(self, lang = "eng"):        # the lang parameter should match one of the languages for which we have articles in the event.
        """
        return keyword aggregate (tag-cloud) from articles in the event
        @param lang: language for which to compute the keywords
        """
        self.resultType = "keywordAggr"
        self.keywordAggrLang = lang



class RequestEventSourceAggr(RequestEvent):
    def __init__(self):
        """
        get news source distribution of articles in the event
        """
        self.resultType = "sourceExAggr"



class RequestEventDateMentionAggr(RequestEvent):
    def __init__(self):
        """
        get dates that we found mentioned in the event articles and their frequencies
        """
        self.resultType = "dateMentionAggr"



class RequestEventArticleTrend(RequestEvent):
    def __init__(self,
                 lang = mainLangs,
                 page = 1, count = 200,
                 minArticleCosSim = -1,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 0))):
        """
        return trending information for the articles about the event
        @param lang: languages for which to compute the trends
        @param page: page of the articles for which to return information (1, 2, ...)
        @param count: number of articles returned per page (at most 200)
        @param minArticleCosSim: ignore articles that have cos similarity to centroid lower than the specified value (-1 for no limit)
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 200, "at most 200 articles can be returned per call"
        self.resultType = "articleTrend"
        self.articleTrendLang = lang
        self.articleTrendPage = page
        self.articleTrendCount = count
        self.articleTrendMinArticleCosSim = minArticleCosSim
        self.__dict__.update(returnInfo.getParams("articleTrend"))



class RequestEventSimilarEvents(RequestEvent):
    def __init__(self,
                 count = 20,                    #
                 maxDayDiff = sys.maxsize,       # what is the maximum time difference between the similar events and this one
                 addArticleTrendInfo = False,   # add info how the articles in the similar events are distributed over time
                 aggrHours = 6,                 # if similarEventsAddArticleTrendInfo == True then this is the aggregating window
                 includeSelf = False,           # should the info about the event itself be included among the results
                 returnInfo = ReturnInfo()):
        """
        compute and return a list of similar events
        @param count: number of similar events to return (at most 200)
        @param maxDayDiff: find only those events that are at most maxDayDiff days apart from the tested event
        @param addArticleTrendInfo: for the returned events compute how they were trending (intensity of reporting) in different time periods
        @param aggrHours: time span that is used as a unit when computing the trending info
        @param includeSel: include also the tested event in the results (True or False)
        @param returnInfo: what details should be included in the returned information
        """
        assert count <= 200
        self.resultType = "similarEvents"
        self.similarEventsCount = count
        if maxDayDiff != sys.maxsize:
            self.similarEventsMaxDayDiff = maxDayDiff
        self.similarEventsAddArticleTrendInfo = addArticleTrendInfo
        self.similarEventsAggrHours = aggrHours
        self.similarEventsIncludeSelf = includeSelf
        self.__dict__.update(returnInfo.getParams("similarEvents"))



class RequestEventSimilarStories(RequestEvent):
    def __init__(self,
                 count = 20,                # number of similar stories to return
                 source = "concept",        # how to compute similarity. Options: concept, cca
                 lang = ["eng"],            # in which language should be the similar stories
                 maxDayDiff = sys.maxsize,   # what is the maximum time difference between the similar stories and this one
                 returnInfo = ReturnInfo()):
        """
        return a list of similar stories (clusters)
        @param count: number of similar stories to return (at most 200)
        @param source: show is the similarity with other stories computed. Using concepts ('concepts') or CCA ('cca').
        @param lang: in what language(s) should be the returned stories
        @param maxDayDiff: maximum difference in days between the returned stories and the tested event
        @param returnInfo: what details should be included in the returned information
        """
        assert count <= 200
        self.resultType = "similarStories"
        self.similarStoriesCount = count
        self.similarStoriesSource = source
        self.similarStoriesLang = lang
        if maxDayDiff != sys.maxsize:
            self.similarStoriesMaxDayDiff = maxDayDiff
        self.__dict__.update(returnInfo.getParams("similarStories"))



from eventregistry.Base import *
from eventregistry.ReturnInfo import *


class QueryArticle(Query):
    def __init__(self,
                 articleUriOrUriList,
                 requestedResult = None):
        """
        Class for obtaining available info for one or more articles in the Event Registry
        @param articleUriOrUriList: a single article uri or a list of article uris
        @param requestedResult: the information to return as the result of the query. By default return the information about the article
        """
        super(QueryArticle, self).__init__()
        self._setVal("articleUri", articleUriOrUriList)
        self._setVal("action", "getArticle")
        self.setRequestedResult(requestedResult or RequestArticleInfo())


    def _getPath(self):
        return "/json/article"


    @staticmethod
    def queryByUri(articleUriOrUriList):
        """
        obtain information about one or more articles by providing their article uris (newsfeed ids, such as "284017606")
        @param articleUriOrUriList: single article uri or a list of article uris to query
        """
        q = QueryArticle([])
        q.queryParams["articleUri"] = articleUriOrUriList
        return q


    def addRequestedResult(self, requestArticle):
        """
        Add a result type that you would like to be returned. In one QueryArticle you can ask for multiple result types.
        Result types can be the classes that extend RequestArticle base class (see classes below).
        @param requestArticle: an instance of type RequestArticle*. Determines what info should be returned as a result of the query
        """
        assert isinstance(requestArticle, RequestArticle), "QueryArticle class can only accept result requests that are of type RequestArticle"
        self.resultTypeList = [item for item in self.resultTypeList if item.getResultType() != requestArticle.getResultType()]
        self.resultTypeList.append(requestArticle)


    def setRequestedResult(self, requestArticle):
        """
        Set the single result type that you would like to be returned. If some other request type was previously set, it will be overwritten.
        Result types can be the classes that extend RequestArticle base class (see classes below).
        """
        assert isinstance(requestArticle, RequestArticle), "QueryArticle class can only accept result requests that are of type RequestArticle"
        self.resultTypeList = [requestArticle]



class RequestArticle:
    def __init__(self):
        self.resultType = None


    def getResultType(self):
        return self.resultType



class RequestArticleInfo(RequestArticle):
    def __init__(self, returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = -1))):
        """
        return details about the article
        @param returnInfo: what details should be included in the returned information
        """
        self.resultType = "info"
        self.__dict__.update(returnInfo.getParams("info"))



class RequestArticleSimilarArticles(RequestArticle):
    def __init__(self,
                 page = 1,
                 count = 20,
                 lang = ["eng"],
                 limitPerLang = -1,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = -1))):
        """
        return a list of similar articles based on the CCA
        @param page: page of the articles
        @param count: number of articles to return (at most 200)
        @param lang: in which language(s) should be the similar articles
        @param limitPerLang: max number of articles per language to return (-1 for no limit)
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 200, "at most 200 articles can be returned per call"
        self.resultType = "similarArticles"
        self.similarArticlesPage = page
        self.similarArticlesCount = count
        self.similarArticlesLang = lang
        self.similarArticlesLimitPerLang = limitPerLang
        self.__dict__.update(returnInfo.getParams("similarArticles"))



class RequestArticleDuplicatedArticles(RequestArticle):
    def __init__(self,
                 page = 1,
                 count = 20,
                 sortBy = "cosSim", sortByAsc = False,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = -1))):
        """
        return a list of duplicated articles of the current article
        @param page: page of the articles
        @param count: number of articles to return (at most 200)
        @param sortBy: how are the articles sorted. Options: id, date, cosSim, fq, socialScore, facebookShares, twitterShares
        @param sortByAsc: should the results be sorted in ascending order (True) or descending (False)
        @param returnInfo: what details should be included in the returned information
        """
        assert page >= 1, "page has to be >= 1"
        assert count <= 200, "at most 200 articles can be returned per call"
        self.resultType = "duplicatedArticles"
        self.duplicatedArticlesPage = page
        self.duplicatedArticlesCount = count
        self.duplicatedArticlesSortBy = sortBy
        self.duplicatedArticlesSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("duplicatedArticles"))



class RequestArticleOriginalArticle(RequestArticle):
    def __init__(self,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = -1))):
        """
        return the article that is the original of the given article (the current article is a duplicate)
        @param returnInfo: what details should be included in the returned information
        """
        self.resultType = "originalArticle"
        self.__dict__.update(returnInfo.getParams("originalArticle"))



"""
the classes here represent all the types of information that can be returned
from Event Registry requests

the ReturnInfo class specifies all types of these parameters and is needed as
a parameter in all query requests
"""

class ReturnInfoFlagsBase(object):
    """
    base class for the return info types
    """

    def _setFlag(self, name, val, defVal):
        """set the objects property propName if the dictKey key exists in dict and it is not the same as default value defVal"""
        if not hasattr(self, "flags"):
            self.flags = {}
        if val != defVal:
            self.flags[name] = val


    def _getFlags(self, prefix):
        """return the dict of stored flags, where each flag name should be first prefixed using prefix"""
        if not hasattr(self, "flags"):
            self.flags = {}
        dict = {}
        for key in list(self.flags.keys()):
            dict[prefix + key] = self.flags[key]
        return dict


    def _setVal(self, name, val):
        """set value of name to val"""
        if not hasattr(self, "vals"):
            self.vals = {}
        self.vals[name] = val


    def _setVal(self, name, val, defVal):
        """set value of name to val in case the val != defVal"""
        if val == defVal:
            return
        if not hasattr(self, "vals"):
            self.vals = {}
        self.vals[name] = val


    def _getVals(self, prefix):
        """
        return the values in the vals dict
        in case prefix is "", change the first letter of the name to lowercase, otherwise use prefix+name as the new name
        """
        if not hasattr(self, "vals"):
            self.vals = {}
        dict = {}
        for key in list(self.vals.keys()):
            # if no prefix then lower the first letter
            if prefix == "":
                newkey = key[:1].lower() + key[1:] if key else ""
                dict[newkey] = self.vals[key]
            else:
                dict[prefix + key] = self.vals[key]
        return dict


class ArticleInfoFlags(ReturnInfoFlagsBase):
    """"
    What information about an article should be returned by the API call

    @param bodyLen: max length of the article body (use -1 for full body, 0 for empty)
    @param basicInfo: core article information -
    @param title: article title
    @param body: article body
    @param url: article url
    @param eventUri: uri of the event to which the article belongs
    @param concepts: the list of concepts mentioned in the article
    @param categories: the list of categories assigned to the article
    @param videos: the list of videos assigned to the article
    @param image: url to the image associated with the article
    @param socialScore: information about the number of times the article was shared on facebook and linkedin, instagram, ...
    @param location: the geographic location that the event mentioned in the article is about
    @param dates: the dates when the articles was crawled and the date when it was published (based on the rss feed date)
    @param extractedDates: the list of dates found mentioned in the article
    @param duplicateList: the list of articles that are a copy of this article
    @param originalArticle: if the article is a duplicate, this will provide information about the original article
    @param storyUri: uri of the story (cluster) to which the article belongs
    @param details: potential additional details
    """
    def __init__(self,
                 bodyLen = -1,
                 basicInfo = True,
                 title = True,
                 body = True,
                 url = True,
                 eventUri = True,
                 concepts = False,
                 categories = False,
                 videos = False,
                 image = False,
                 socialScore = False,
                 duplicateList = False,
                 originalArticle = False,
                 location = False,
                 dates = False,
                 extractedDates = False,
                 storyUri = False,
                 details = False):
        self._setVal("ArticleBodyLen", bodyLen, 300)
        self._setFlag("IncludeArticleBasicInfo", basicInfo, True)
        self._setFlag("IncludeArticleTitle", title, True)
        self._setFlag("IncludeArticleBody", body, True)
        self._setFlag("IncludeArticleUrl", url, True)
        self._setFlag("IncludeArticleEventUri", eventUri, True)
        self._setFlag("IncludeArticleConcepts", concepts, False)
        self._setFlag("IncludeArticleCategories", categories, False)
        self._setFlag("IncludeArticleVideos", videos, False)
        self._setFlag("IncludeArticleImage", image, False)
        self._setFlag("IncludeArticleSocialScore", socialScore, False)
        self._setFlag("IncludeArticleLocation", location, False)
        self._setFlag("IncludeArticleDates", dates, False)
        self._setFlag("IncludeArticleExtractedDates", extractedDates, False)
        self._setFlag("IncludeArticleDuplicateList", duplicateList, False)
        self._setFlag("IncludeArticleOriginalArticle", originalArticle, False)
        self._setFlag("IncludeArticleStoryUri", storyUri, False)
        self._setFlag("IncludeArticleDetails", details, False)



class StoryInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a story (cluster of articles) should be returned by the API call

    @param basicStats: core stats about the story
    @param location: geographic location that the story is about
    @param categories: categories associated with the story
    @param title: title of the story
    @param summary: summary of the story
    @param medoidArticle: the article that is closest to the center of the cluster of articles assigned to the story
    @param commonDates: dates that were frequently identified in the articles belonging to the story
    @param socialScore: score computed based on how frequently the articles in the story were shared on social media
    @param imageCount: number of images to be returned for a story
    @param flags: various binary flags related to the story
    """
    def __init__(self,
                 basicStats = True,
                 location = True,
                 categories = False,
                 date = False,
                 concepts = False,
                 title = False,
                 summary = False,
                 medoidArticle = False,
                 commonDates = False,
                 socialScore = False,
                 details = False,
                 flags = False,
                 imageCount = 0):
        self._setFlag("IncludeStoryBasicStats", basicStats, True)
        self._setFlag("IncludeStoryLocation", location, True)

        self._setFlag("IncludeStoryCategories", categories, False)
        self._setFlag("IncludeStoryDate", date, False)
        self._setFlag("IncludeStoryConcepts", concepts, False)
        self._setFlag("IncludeStoryTitle", title, False)
        self._setFlag("IncludeStorySummary", summary, False)
        self._setFlag("IncludeStoryMedoidArticle", medoidArticle, False)
        self._setFlag("IncludeStoryCommonDates", commonDates, False)
        self._setFlag("IncludeStorySocialScore", socialScore, False)
        self._setFlag("IncludeStoryDetails", details, False)
        self._setVal("StoryImageCount", imageCount, 0)



class EventInfoFlags(ReturnInfoFlagsBase):
    """
    What information about an event should be returned by the API call

    @param title: return the title of the event
    @param summary: return the summary of the event
    @param articleCounts: return the number of articles that are assigned to the event
    @param concepts: return information about the main concepts related to the event
    @param categories: return information about the categories related to the event
    @param location: return the location where the event occured
    @param date: return information about the date of the event
    @param commonDates: return the dates that were commonly found in the articles about the event
    @param stories: return the list of stories (clusters) that are about the event
    @param socialScore: score computed based on how frequently the articles in the event were shared on social media
    @param imageCount: number of images to be returned for an event
    @param flags: various binary flags related to the event
    """
    def __init__(self,
                 title = True,
                 summary = True,
                 articleCounts = True,
                 concepts = True,
                 categories = True,
                 location = True,
                 date = True,
                 commonDates = False,
                 stories = False,
                 socialScore = False,
                 details = False,
                 flags = False,
                 imageCount = 0):
        self._setFlag("IncludeEventTitle", title, True)
        self._setFlag("IncludeEventSummary", summary, True)
        self._setFlag("IncludeEventArticleCounts", articleCounts, True)
        self._setFlag("IncludeEventConcepts", concepts, True)
        self._setFlag("IncludeEventCategories", categories, True)
        self._setFlag("IncludeEventLocation", location, True)
        self._setFlag("IncludeEventDate", date, True)

        self._setFlag("IncludeEventCommonDates", commonDates, False)
        self._setFlag("IncludeEventStories", stories, False)
        self._setFlag("IncludeEventSocialScore", socialScore, False)
        self._setFlag("IncludeEventDetails", details, False)
        self._setVal("EventImageCount", imageCount, 0)



class SourceInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a news source should be returned by the API call

    @param title: title of the news source
    @param description: description of the news source
    @param location: geographic location of the news source
    @param ranking: a set of rankings for the news source
    @param articleCount: the number of articles from this news source that are stored in Event Registry
    @param sourceGroups: info about the names of the source groups to which the source belongs to
    @param flags: various binary flags related to the news source
    """
    def __init__(self,
                 title = True,
                 description = False,
                 location = False,
                 ranking = False,
                 image = False,
                 articleCount = False,
                 sourceGroups = False,
                 details = False,
                 flags = False):
        self._setFlag("IncludeSourceTitle", title, True)

        self._setFlag("IncludeSourceDescription", description, False)
        self._setFlag("IncludeSourceLocation", location, False)
        self._setFlag("IncludeSourceRanking", ranking, False)
        self._setFlag("IncludeSourceImage", image, False)
        self._setFlag("IncludeSourceArticleCount", articleCount, False)
        self._setFlag("IncludeSourceSourceGroups", sourceGroups, False)
        self._setFlag("IncludeSourceDetails", details, False)



class CategoryInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a category should be returned by the API call

    @param parentUri: uri of the parent category
    @param childrenUris: the list of category uris that are children of the category
    @param trendingScore: information about how the category is currently trending. The score is computed as Pearson residual by comparing the trending of the category in last 2 days compared to last 14 days
    @param trendingHistory: information about the number of times articles were assigned to the category in last 30 days
    @param trendingSource: source of information to be used when computing the trending score for a category. Relevant only if CategoryInfoFlags.trendingScore == True or CategoryInfoFlags.trendingHistory == True. Valid options: news, social
    @param flags: various binary flags related to the category
    @type trendingSource: string | list
    """
    def __init__(self,
                 parentUri = False,
                 childrenUris = False,
                 trendingScore = False,
                 trendingHistory = False,
                 details = False,
                 trendingSource = "news",
                 flags = False):
        self._setFlag("IncludeCategoryParentUri", parentUri, False)
        self._setFlag("IncludeCategoryChildrenUris", childrenUris, False)
        self._setFlag("IncludeCategoryTrendingScore", trendingScore, False)
        self._setFlag("IncludeCategoryTrendingHistory", trendingHistory, False)
        self._setFlag("IncludeCategoryDetails", details, False)
        self._setVal("CategoryTrendingSource", trendingSource, "news")



class ConceptInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a concept should be returned by the API call

    @param type: which types of concepts should be provided in events, stories, ... Options: person, loc, org, wiki (non-entities), concepts (=person+loc+org+wiki), conceptClass, conceptFolder
    @param lang: in which languages should be the labels for provided concepts
    @param label: return label(s) of the concept
    @param synonyms: return concept synonyms (if any)
    @param image: provide an image associated with the concept
    @param description: description of the concept
    @param conceptClassMembership: provide a list of concept classes where the concept is a member
    @param conceptClassMembership: provide a list of concept classes and their parents where the concept is a member
    @param trendingScore: information about how the concept is currently trending. The score is computed as Pearson residual by comparing the trending of the concept in last 2 days compared to last 14 days
    @param trendingHistory: information about the number of times articles were assigned to the concept in last 30 days
    @param totalCount: the total number of times the concept appeared in the news articles
    @param trendingSource: source of information to be used when computing the trending score for a concept. Relevant only if ConceptInfoFlags.trendingScore == True or ConceptInfoFlags.trendingHistory == True. Valid options: news, social
    @param flags: various binary flags related to the concept
    @type conceptType: str | list
    @type conceptLang: str | list
    @type trendingSource: string | list
    """
    def __init__(self,
                 type = "concepts",
                 lang = "eng",
                 label = True,
                 synonyms = False,
                 image = False,
                 description = False,
                 details = False,
                 conceptClassMembership = False,
                 conceptClassMembershipFull = False,
                 trendingScore = False,
                 trendingHistory = False,
                 trendingSource = "news",
                 totalCount = False,
                 maxConceptsPerType = 20,
                 flags = False):
        self._setVal("ConceptType", type, "concepts")
        self._setVal("ConceptLang", lang, "eng")
        self._setFlag("IncludeConceptLabel", label, True)
        self._setFlag("IncludeConceptSynonyms", synonyms, False)
        self._setFlag("IncludeConceptImage", image, False)
        self._setFlag("IncludeConceptDescription", description, False)
        self._setFlag("IncludeConceptDetails", details, False)
        self._setFlag("IncludeConceptConceptClassMembership", conceptClassMembership, False)
        self._setFlag("IncludeConceptConceptClassMembershipFull", conceptClassMembershipFull, False)
        self._setFlag("IncludeConceptTrendingScore", trendingScore, False)
        self._setFlag("IncludeConceptTrendingHistory", trendingHistory, False)
        self._setFlag("IncludeConceptTotalCount", totalCount, False)
        self._setVal("ConceptTrendingSource", trendingSource, "news")
        self._setVal("MaxConceptsPerType", maxConceptsPerType, 20)



class LocationInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a geographic location should be returned by the API call
    Locations are sub-types of concepts so this information is always provided as a "location" property in concept information
    country* flags are taken into account when the locations represent countries. Similarly place* flags are relevant when the location is a place (city, area, ...)
    @param label: return label of the place/country
    @param wikiUri: return wiki url of the place/country
    @param geoNamesId: return geonames id for the place/country
    @param population: return the population of the place/country
    @param geoLocation: return geographic coordinates of the place/country

    @param countryArea: return geographic area of the country
    @param countryContinent: return continent where the country is located

    @param placeFeatureCode: return the geonames feature code of the place
    @param placeCountry: return information about the country where the place is located
    """
    def __init__(self,
                 label = True,
                 wikiUri = False,
                 geoNamesId = False,
                 population = False,
                 geoLocation = False,

                 countryArea = False,
                 countryDetails = False,
                 countryContinent = False,

                 placeFeatureCode = False,
                 placeCountry = True):
        self._setFlag("IncludeLocationLabel", label, True)
        self._setFlag("IncludeLocationWikiUri", wikiUri, False)
        self._setFlag("IncludeLocationGeoNamesId", geoNamesId, False)
        self._setFlag("IncludeLocationPopulation", population, False)
        self._setFlag("IncludeLocationGeoLocation", geoLocation, False)

        self._setFlag("IncludeLocationCountryArea", countryArea, False)
        self._setFlag("IncludeLocationCountryDetails", countryDetails, False)
        self._setFlag("IncludeLocationCountryContinent", countryContinent, False)

        self._setFlag("IncludeLocationPlaceFeatureCode", placeFeatureCode, False)
        self._setFlag("IncludeLocationPlaceCountry", placeCountry, True)



class ConceptClassInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a concept class should be returned by the API call

    @param parentLabels: return the list of labels of the parent concept classes
    @param concepts: return the list of concepts assigned to the concept class
    @param details: return additional details about the concept class
    @param flags: various binary flags related to the concept class
    """
    def __init__(self,
                 parentLabels = True,
                 concepts = False,
                 details = False,
                 flags = False):
        self._setFlag("IncludeConceptClassParentLabels", parentLabels, True)
        self._setFlag("IncludeConceptClassConcepts", concepts, False)
        self._setFlag("IncludeConceptClassDetails", details, False)



class ConceptFolderInfoFlags(ReturnInfoFlagsBase):
    """
    What information about a concept folder should be returned by the API call

    @param definition: return the complete definition of the concept folder
    @param owner: return information about the owner of the concept folder
    @param details: return additional details about the concept folder
    @param flags: various binary flags related to the concept folder
    """
    def __init__(self,
                 definition = False,
                 owner = False,
                 details = False,
                 flags = False):
        self._setFlag("IncludeConceptFolderDefinition", definition, False)
        self._setFlag("IncludeConceptFolderOwner", owner, False)
        self._setFlag("IncludeConceptFolderDetails", details, False)



class ReturnInfo:
    """
    ReturnInfo specifies what content should be returned for each possible returned object type

    @param articleInfo: what details about the articles should be returned
    @param eventInfo: what details about the event should be returned
    @param sourceInfo: what details about the article's news source should be returned
    @param storyInfo: what details about the stories (clusters) should be returned
    @param categoryInfo: what details about the categories should be returned
    @param conceptInfo: what details about the concepts should be returned
    @param locationInfo: what details about the locations should be returned (locations are sub-types of concepts so their information will be a property inside the concept information)
    @param conceptClassInfo: what details about the concept classes should be returned (concept classes are sub-types of concepts so their information will be a property inside the concept information)
    @param conceptFolderInfo: what details about the concept folders should be returned (concept folders are sub-types of concepts so their information will be a property inside the concept information)

    @type articleInfo: ArticleInfoFlags
    @type eventInfo: EventInfoFlags
    @type sourceInfo: SourceInfoFlags
    @type storyInfo: StoryInfoFlags
    @type categoryInfo: CategoryInfoFlags
    @type conceptInfo: ConceptInfoFlags
    @type locationInfo: LocationInfoFlags
    @type conceptClassInfo: ConceptClassInfoFlags
    @type conceptFolderInfo: ConceptFolderInfoFlags
    """
    def __init__(self,
                 articleInfo = ArticleInfoFlags(),
                 eventInfo = EventInfoFlags(),
                 sourceInfo = SourceInfoFlags(),
                 categoryInfo = CategoryInfoFlags(),
                 conceptInfo = ConceptInfoFlags(),
                 locationInfo = LocationInfoFlags(),
                 storyInfo = StoryInfoFlags(),
                 conceptClassInfo = ConceptClassInfoFlags(),
                 conceptFolderInfo = ConceptFolderInfoFlags()):
        assert isinstance(articleInfo, ArticleInfoFlags)
        assert isinstance(eventInfo, EventInfoFlags)
        assert isinstance(sourceInfo, SourceInfoFlags)
        assert isinstance(categoryInfo, CategoryInfoFlags)
        assert isinstance(conceptInfo, ConceptInfoFlags)
        assert isinstance(locationInfo, LocationInfoFlags)
        assert isinstance(storyInfo, StoryInfoFlags)
        assert isinstance(conceptClassInfo, ConceptClassInfoFlags)
        assert isinstance(conceptFolderInfo, ConceptFolderInfoFlags)
        self.articleInfo = articleInfo
        self.eventInfo = eventInfo
        self.sourceInfo = sourceInfo
        self.categoryInfo = categoryInfo
        self.conceptInfo = conceptInfo
        self.locationInfo = locationInfo
        self.storyInfo = storyInfo
        self.conceptClassInfo = conceptClassInfo
        self.conceptFolderInfo = conceptFolderInfo


    def getParams(self, prefix = ""):
        dict = {}
        dict.update(self.articleInfo._getFlags(prefix == "" and "article" or prefix))
        dict.update(self.eventInfo._getFlags(prefix == "" and "event" or prefix))
        dict.update(self.sourceInfo._getFlags(prefix == "" and "source" or prefix))
        dict.update(self.conceptInfo._getFlags(prefix == "" and "concept" or prefix))
        dict.update(self.categoryInfo._getFlags(prefix == "" and "category" or prefix))
        dict.update(self.locationInfo._getFlags(prefix == "" and "location" or prefix))
        dict.update(self.storyInfo._getFlags(prefix == "" and "story" or prefix))
        dict.update(self.conceptClassInfo._getFlags(prefix == "" and "conceptClass" or prefix))
        dict.update(self.conceptFolderInfo._getFlags(prefix == "" and "conceptFolder" or prefix))
        dict.update(self.articleInfo._getVals(prefix))
        dict.update(self.eventInfo._getVals(prefix))
        dict.update(self.sourceInfo._getVals(prefix))
        dict.update(self.conceptInfo._getVals(prefix))
        dict.update(self.categoryInfo._getVals(prefix))
        dict.update(self.locationInfo._getVals(prefix))
        dict.update(self.storyInfo._getVals(prefix))
        dict.update(self.conceptClassInfo._getVals(prefix))
        dict.update(self.conceptFolderInfo._getVals(prefix))
        return dict




"""
the GetEventForText class can be used to find the event(s) that best matches the given input text.
The request is performed asynchronously.

Note: The functionality can only be used to find the events in the last 5 days. Older events cannot
be matched in this way

Return info from the compute() method is in the form:
[
    {
        "cosSim": 0.07660648086468567,
        "eventUri": "4969",
        "storyUri": "eng-af6ce79f-cb91-4010-8ddf-7ad924bc5638-40591"
    },
    {
        "cosSim": 0.05851939237670918,
        "eventUri": "5157",
        "storyUri": "eng-af6ce79f-cb91-4010-8ddf-7ad924bc5638-56286"
    },
    ...
]

where
* cosSim represents the cosine similarity of the document to the cluster
* eventUri is the uri of the corresponding event in the Event Registry
* storyUri is the uri of the story in the Event Registry
You can use QueryEvent or QueryStory to obtain more information about these events/stories
"""

from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class GetEventForText(QueryParamsBase):
    def __init__(self,
                 eventRegistry,             # instance of EventRegistry class
                 nrOfEventsToReturn = 5):   # number of events to return for the given text
        QueryParamsBase.__init__(self)
        self._er = eventRegistry
        self._nrOfEventsToReturn = nrOfEventsToReturn;


    def compute(self,
                text,           # text for which to find the most similar event
                lang = "eng"):  # language in which the text is written
        """
        compute the list of most similar events for the given text
        """
        params = { "lang": lang, "text": text, "topClustersCount": self._nrOfEventsToReturn }
        res = self._er.jsonRequest("/json/getEventForText/enqueueRequest", params)

        requestId = res["requestId"]
        for i in range(10):
            time.sleep(1)   # sleep for 1 second to wait for the clustering to perform computation
            res = self._er.jsonRequest("/json/getEventForText/testRequest", { "requestId": requestId })
            if isinstance(res, list) and len(res) > 0:
                return res
        return None



"""
trending information is computed by comparing how frequently are individual concepts/categories
mentioned in the articles. By default trends are computed by comparing the total number of mentions of a concept/category
in the last two days compared to the number of mentions in the two weeks before. The trend for each concept/category
is computed as the Pearson residual. The returned concepts/categories are the ones that have the highest residual.
"""
from eventregistry.Base import *
from eventregistry.ReturnInfo import *


class TrendsBase(QueryParamsBase):
    def _getPath(self):
        return "/json/trends"


class GetTrendingConcepts(TrendsBase):
    def __init__(self,
                 source = "news",
                 count = 20,
                 conceptType = ["person", "org", "loc"],
                 returnInfo = ReturnInfo()):
        """
        get currently top trending concepts
        @param source: source information from which to compute top trends. Options: "news", "social"
        @param count: number of top trends to return
        @param conceptType: which types of concepts are we interested in
        @param returnInfo: what details should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getTrendingConcepts")
        self._setVal("source", source)
        self._setVal("conceptCount", count)
        self._setVal("conceptType", conceptType)
        self._update(returnInfo.getParams())



class GetTrendingCategories(TrendsBase):
    def __init__(self,
                 source = "news",
                 count = 20,
                 returnInfo = ReturnInfo()):
        """
        get currently top trending categories
        @param source: source information from which to compute top trends. Options: "news", "social"
        @param count: number of top trends to return
        @param returnInfo: what details should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getTrendingCategories")
        self._setVal("source", source)
        self._setVal("categoryCount", count)
        self._update(returnInfo.getParams())



class GetTrendingCustomItems(TrendsBase):
    def __init__(self,
                 count = 20,
                 returnInfo = ReturnInfo()):
        """
        get currently top trending items for which the users provided the data
        this data can be stock prices, energy prices, etc...
        @param count: number of top trends to return
        @param returnInfo: what details should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getTrendingCustom")
        self._setVal("conceptCount", count)
        self._update(returnInfo.getParams())



class GetTrendingConceptGroups(TrendsBase):
    def __init__(self,
                 source = "news",
                 count = 20,
                 returnInfo = ReturnInfo()):
        """
        get currently top trending groups of concepts
        a group can be identified by the concept type or by a concept class uri
        @param source: source information from which to compute top trends. Options: "news", "social"
        @param count: number of top trends to return
        @param returnInfo: what details should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getConceptTrendGroups")
        self._setVal("source", source)
        self._setVal("conceptCount", count)
        self._update(returnInfo.getParams())


    def getConceptTypeGroups(self, types = ["person", "org", "loc"]):
        """request trending of concepts of specified types"""
        self._setVal("conceptType", types)


    def getConceptClassUris(self, conceptClassUris):
        """request trending of concepts assigned to the specified concept classes"""
        self._setVal("conceptClassUri", conceptClassUris)




from .Base import QueryParamsBase, QueryItems
import six


class _QueryCore(object):
    def __init__(self):
        self._queryObj = {}


    def getQuery(self):
        return self._queryObj


    def setQueryParam(self, paramName, val):
        self._queryObj[paramName] = val


    def _setValIfNotDefault(self, propName, value, defVal):
        if value != defVal:
            self._queryObj[propName] = value



class BaseQuery(_QueryCore):
    def __init__(self,
                 keyword = None,
                 conceptUri = None,
                 categoryUri = None,
                 sourceUri = None,
                 locationUri = None,
                 lang = None,
                 dateStart = None,
                 dateEnd = None,
                 dateMention = None,
                 sourceLocationUri = None,
                 sourceGroupUri = None,
                 keywordLoc = "body",
                 minMaxArticlesInEvent = None,
                 exclude = None):
        """
        @param keyword: keyword(s) to query. Either None, string or QueryItems
        @param conceptUri: concept(s) to query. Either None, string or QueryItems
        @param sourceUri: source(s) to query. Either None, string or QueryItems
        @param locationUri: location(s) to query. Either None, string or QueryItems
        @param categoryUri: categories to query. Either None, string or QueryItems
        @param lang: language(s) to query. Either None, string or QueryItems
        @param dateStart: starting date. Either None, string or date or datetime
        @param dateEnd: ending date. Either None, string or date or datetime
        @param dateMention: search by mentioned dates - Either None, string or date or datetime or a list of these types
        @param sourceLocationUri: find content generated by news sources at the specified geographic location - can be a city URI or a country URI. Multiple items can be provided using a list
        @param sourceGroupUri: a single or multiple source group URIs. A source group is a group of news sources, commonly defined based on common topic or importance
        @param keywordLoc: where should we look when searching using the keywords provided by "keyword" parameter. "body" (default), "title", or "body,title"
        @param minMaxArticlesInEvent: a tuple containing the minimum and maximum number of articles that should be in the resulting events. Parameter relevant only if querying events
        @param exclude: a instance of BaseQuery, CombinedQuery or None. Used to filter out results matching the other criteria specified in this query
        """
        super(BaseQuery, self).__init__()

        self._setQueryArrVal("keyword", keyword)
        self._setQueryArrVal("conceptUri", conceptUri)
        self._setQueryArrVal("categoryUri", categoryUri)
        self._setQueryArrVal("sourceUri", sourceUri)
        self._setQueryArrVal("locationUri", locationUri)
        self._setQueryArrVal("lang", lang)

        # starting date of the published articles (e.g. 2014-05-02)
        if dateStart != None:
            self._queryObj["dateStart"] = QueryParamsBase.encodeDate(dateStart)
        # ending date of the published articles (e.g. 2014-05-02)
        if dateEnd != None:
            self._queryObj["dateEnd"] = QueryParamsBase.encodeDate(dateEnd)

        # mentioned date detected in articles (e.g. 2014-05-02)
        if dateMention != None:
            if isinstance(dateMention, list):
                self._queryObj["dateMention"] = [QueryParamsBase.encodeDate(d) for d in dateMention]
            else:
                self._queryObj["dateMention"] = QueryParamsBase.encodeDate(dateMention)

        self._setQueryArrVal("sourceLocationUri", sourceLocationUri)
        self._setQueryArrVal("sourceGroupUri", sourceGroupUri)
        if keywordLoc != "body":
            self._queryObj["keywordLoc"] = keywordLoc

        if minMaxArticlesInEvent != None:
            assert isinstance(minMaxArticlesInEvent, tuple), "minMaxArticlesInEvent parameter should either be None or a tuple with two integer values"
            self._queryObj["minArticlesInEvent"] = minMaxArticlesInEvent[0]
            self._queryObj["maxArticlesInEvent"] = minMaxArticlesInEvent[1]

        if exclude != None:
            assert isinstance(exclude, (CombinedQuery, BaseQuery)), "exclude parameter was not a CombinedQuery or BaseQuery instance"
            self._queryObj["$not"] = exclude.getQuery()


    def _setQueryArrVal(self, propName, value):
        # by default we have None - so don't do anything
        if value is None:
            return
        # if we have an instance of QueryItems then apply it
        if isinstance(value, QueryItems):
            self._queryObj[propName] = { value.getOper(): value.getItems() }

        # if we have a string value, just use it
        elif isinstance(value, six.string_types):
            self._queryObj[propName] = value
        # there should be no other valid types
        else:
            assert False, "Parameter '%s' was of unsupported type. It should either be None, a string or an instance of QueryItems" % (propName)



class CombinedQuery(_QueryCore):
    def __init__(self):
        super(CombinedQuery, self).__init__()


    @staticmethod
    def AND(queryArr,
            exclude = None):
        """
        create a combined query with multiple items on which to perform an AND operation
        @param queryArr: a list of items on which to perform an AND operation. Items can be either a CombinedQuery or BaseQuery instances.
        @param exclude: a instance of BaseQuery, CombinedQuery or None. Used to filter out results matching the other criteria specified in this query
        """
        assert isinstance(queryArr, list), "provided argument as not a list"
        assert len(queryArr) > 0, "queryArr had an empty list"
        q = CombinedQuery()
        q.setQueryParam("$and", [])
        for item in queryArr:
            assert isinstance(item, (CombinedQuery, BaseQuery)), "item in the list was not a CombinedQuery or BaseQuery instance"
            q.getQuery()["$and"].append(item.getQuery())
        if exclude != None:
            assert isinstance(exclude, (CombinedQuery, BaseQuery)), "exclude parameter was not a CombinedQuery or BaseQuery instance"
            q.setQueryParam("$not", exclude.getQuery())
        return q


    @staticmethod
    def OR(queryArr,
           exclude = None):
        """
        create a combined query with multiple items on which to perform an OR operation
        @param queryArr: a list of items on which to perform an OR operation. Items can be either a CombinedQuery or BaseQuery instances.
        @param exclude: a instance of BaseQuery, CombinedQuery or None. Used to filter out results matching the other criteria specified in this query
        """
        assert isinstance(queryArr, list), "provided argument as not a list"
        assert len(queryArr) > 0, "queryArr had an empty list"
        q = CombinedQuery()
        q.setQueryParam("$or", [])
        for item in queryArr:
            assert isinstance(item, (CombinedQuery, BaseQuery)), "item in the list was not a CombinedQuery or BaseQuery instance"
            q.getQuery()["$or"].append(item.getQuery())
        if exclude != None:
            assert isinstance(exclude, (CombinedQuery, BaseQuery)), "exclude parameter was not a CombinedQuery or BaseQuery instance"
            q.setQueryParam("$not", exclude.getQuery())
        return q



class ComplexArticleQuery(_QueryCore):
    def __init__(self,
                 query,
                 isDuplicateFilter = "keepAll",
                 hasDuplicateFilter = "keepAll",
                 eventFilter = "keepAll"):
        """
        create an article query using a complex query
        @param query: an instance of CombinedQuery or BaseQuery to use to find articles that match the conditions
        @param isDuplicateFilter: some articles can be duplicates of other articles. What should be done with them. Possible values are:
                "skipDuplicates" (skip the resulting articles that are duplicates of other articles)
                "keepOnlyDuplicates" (return only the duplicate articles)
                "keepAll" (no filtering, default)
        @param hasDuplicateFilter: some articles are later copied by others. What should be done with such articles. Possible values are:
                "skipHasDuplicates" (skip the resulting articles that have been later copied by others)
                "keepOnlyHasDuplicates" (return only the articles that have been later copied by others)
                "keepAll" (no filtering, default)
        @param eventFilter: some articles describe a known event and some don't. This filter allows you to filter the resulting articles based on this criteria.
                Possible values are:
                "skipArticlesWithoutEvent" (skip articles that are not describing any known event in ER)
                "keepOnlyArticlesWithoutEvent" (return only the articles that are not describing any known event in ER)
                "keepAll" (no filtering, default)
        """
        super(ComplexArticleQuery, self).__init__()

        assert isinstance(query, (CombinedQuery, BaseQuery)), "query parameter was not a CombinedQuery or BaseQuery instance"
        self._queryObj["$query"] = query.getQuery()
        filter = {}
        if (isDuplicateFilter != "keepAll"):
            filter["isDuplicate"] = isDuplicateFilter
        if (hasDuplicateFilter != "keepAll"):
            filter["hasDuplicate"] = hasDuplicateFilter
        if (eventFilter != "keepAll"):
            filter["hasEvent"] = eventFilter
        if len(filter) > 0:
            self._queryObj["$filter"] = filter



class ComplexEventQuery(_QueryCore):
    def __init__(self, query):
        """
        create an event query using a complex query
        @param query: an instance of CombinedQuery or BaseQuery to use to find events that match the conditions
        """
        super(ComplexEventQuery, self).__init__()

        assert isinstance(query, (CombinedQuery, BaseQuery)), "query parameter was not a CombinedQuery or BaseQuery instance"
        self._queryObj["$query"] = query.getQuery()



"""
utility classes for Event Registry
"""

import six, warnings, os, sys, re, datetime, time


mainLangs = ["eng", "deu", "zho", "slv", "spa"]
allLangs = [ "eng", "deu", "spa", "cat", "por", "ita", "fra", "rus", "ara", "tur", "zho", "slv", "hrv", "srp" ]
conceptTypes = ["loc", "person", "org", "keyword", "wiki", "conceptClass", "conceptFolder"]


def deprecated(func):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emmitted
    when the function is used."""

    def new_func(*args, **kwargs):
        warnings.simplefilter('always', DeprecationWarning) #turn off filter
        warnings.warn("Call to deprecated function {}.".format(func.__name__), category=DeprecationWarning, stacklevel=2)
        warnings.simplefilter('default', DeprecationWarning) #reset filter
        return func(*args, **kwargs)

    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func


invalidCharRe = re.compile(r"[\x00-\x08]|\x0b|\x0c|\x0e|\x0f|[\x10-\x19]|[\x1a-\x1f]", re.IGNORECASE)
def removeInvalidChars(text):
    return invalidCharRe.sub("", text)


def tryParseInt(s, base=10, val=None):
    try:
        return int(s, base)
    except ValueError:
        return val


class Struct(object):
    """
    helper class for converting dict to a native python object
    instead of a["b"]["c"] we can write a.b.c
    """
    def __init__(self, data):
        for name, value in data.items():
            setattr(self, name, self._wrap(value))


    def _wrap(self, value):
        if isinstance(value, (tuple, list, set, frozenset)):
            return type(value)([self._wrap(v) for v in value])
        else:
            return Struct(value) if isinstance(value, dict) else value


    # does the object have the key
    def has(self, key):
        return hasattr(self, key)



def createStructFromDict(data):
    """method to convert a list or dict to a native python object"""
    if isinstance(data, list):
        return type(data)([createStructFromDict(v) for v in data])
    else:
        return Struct(data)


class QueryItems:
    _AND = "$and"
    _OR = "$or"
    _Undef = None

    def __init__(self, oper, items):
        self._oper = oper
        self._items = items;

    @staticmethod
    def AND(items):
        return QueryItems(QueryItems._AND, items)

    @staticmethod
    def OR(items):
        return QueryItems(QueryItems._OR, items)

    def getOper(self):
        return self._oper

    def getItems(self):
        return self._items


class QueryParamsBase(object):
    """
    Base class for Query and AdminQuery
    used for storing parameters for a query. Parameter values can either be
    simple values (set by _setVal()) or an array of values (set by multiple
    calls to _addArrayVal() method)
    """
    def __init__(self):
        self.queryParams = {}


    @staticmethod
    def copy(obj):
        assert isinstance(obj, QueryParamsBase)
        ret = QueryParamsBase()
        ret.queryParams = dict(obj.queryParams)
        return ret


    @staticmethod
    def encodeDate(val):
        """encode val that can be a date in different forms as a date that can be sent to Er"""
        if isinstance(val, datetime.datetime):
            return val.date().isoformat()
        elif isinstance(val, datetime.date):
            return val.isoformat()
        elif isinstance(val, six.string_types):
            assert re.match("^\d{4}-\d{2}-\d{2}$", val), "date value '%s' was not provided in the 'YYYY-MM-DD' format" % (val)
            return val
        raise AssertionError("date was not in the expected format")


    @staticmethod
    def encodeDateTime(val):
        """encode datetime into UTC ISO format which can be sent to ER"""
        if isinstance(val, datetime.datetime):
            # if we have a datetime in some tz, we convert it first to UTC
            if val.utcoffset() != None:
                import pytz
                val = val.astimezone(pytz.utc)
            return val.isoformat()
        elif isinstance(val, six.string_types):
            assert re.match("^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(\.\d+)?$", val), "datetime value '%s' was not provided in the 'YYYY-MM-DDTHH:MM:SS.SSSS' format" % (val)
            return val
        raise AssertionError("datetime was not in the recognizable data type. Use datetime or string in ISO format")


    def _clearVal(self, propName):
        """remove the value of a property propName (if existing)"""
        if propName in self.queryParams:
            del self.queryParams[propName]


    def _hasVal(self, propName):
        """do we have in the query property named propName"""
        return propName in self.queryParams


    def _setVal(self, propName, val):
        """set a value of a property in the query"""
        if isinstance(val, six.string_types):
            # in python 2 we need to first encode, before removing the invalid characters
            if six.PY2:
                val = val.encode("utf8")
            val = removeInvalidChars(val)
        self.queryParams[propName] = val


    def _setValIfNotDefault(self, propName, val, defVal):
        """set to queryParams property propName to val if val != defVal"""
        if val != defVal:
            self._setVal(propName, val)


    def _setDateVal(self, propName, val):
        """set a property value that represents date. Value can be string in YYYY-MM-DD format, datetime.date or datetime.datetime"""
        encodedVal = self.encodeDate(val)
        self._setVal(propName, encodedVal)


    def _addArrayVal(self, propName, val):
        """add a value to an array of values for a property"""
        if isinstance(val, six.string_types):
            # in python 2 we need to first encode, before removing the invalid characters
            if six.PY2:
                val = val.encode("utf8")
            val = removeInvalidChars(val)
        if propName not in self.queryParams:
            self.queryParams[propName] = []
        self.queryParams[propName].append(val)


    def _update(self, object):
        self.queryParams.update(object)


    def _getQueryParams(self):
        """return the parameters."""
        return dict(self.queryParams)



class Query(QueryParamsBase):
    def __init__(self):
        QueryParamsBase.__init__(self)
        self.resultTypeList = []


    def clearRequestedResults(self):
        self.resultTypeList = []


    def _getQueryParams(self):
        """encode the request."""
        allParams = {}
        if len(self.resultTypeList) == 0:
            raise ValueError("The query does not have any result type specified. No sense in performing such a query")
        allParams.update(self.queryParams)
        for request in self.resultTypeList:
            allParams.update(request.__dict__)
        # all requests in resultTypeList have "resultType" so each call to .update() overrides the previous one
        # since we want to store them all we have to add them here:
        allParams["resultType"] = [request.__dict__["resultType"] for request in self.resultTypeList]
        return allParams


    def _setQueryArrVal(self, value, propName, propOperName, defaultOperName):
        """
        parse the value "value" and use it to set the property propName and the operator with name propOperName
        @param value: None, string, QueryItems or list. Values to be set using property name propName
        @param propOperName: property to set containing the "and" or "or". Relevant only if multiple items are provided in "value". Can be None if only one value is possible
        @param defaultOperName: which operator should be used in case "value" is a list. If a list, we will print also a warning to suggest use of QueryItems
        """
        # by default we have None - so don't do anything
        if value is None or value == "":
            return
        # if we have an instance of QueryItems then apply it
        if isinstance(value, QueryItems):
            self.queryParams[propName] = value.getItems()
            # if we need to specify the operator for the property
            if propOperName != None:
                self.queryParams[propOperName] = value.getOper().replace("$", "")
            # if the user specified the QueryItems class but used the invalid operator type then raise an error
            assert propOperName != None or value.getOper().replace("$", "") == defaultOperName, "An invalid operator type '%s' was used for property '%s'" % (value.getOper().replace("$", ""), propName)

        # if we have a string value, just use it
        elif isinstance(value, six.string_types):
            self.queryParams[propName] = value

        # if we have a list, set it, but also weport
        elif isinstance(value, list):
            self.queryParams[propName] = value
            # if we need to specify the operator for the property
            if propOperName != None:
                self.queryParams[propOperName] = defaultOperName
                if len(value) > 1:
                    print("Warning: The value of parameter '%s' was provided as a list and '%s' operator was used implicitly between the items. We suggest specifying the list using the QueryItems.AND() or QueryItems.OR() to ensure the appropriate operator is used." % (propName, defaultOperName))

        # there should be no other valid types
        else:
            assert False, "Parameter '%s' was of unsupported type. It should either be None, a string or an instance of QueryItems" % (propName)


"""
provides classes needed to identify concepts or categories that trend the most with a concept, category or a custom time series
"""

import json, six
from eventregistry.Base import *
from eventregistry.ReturnInfo import *
from eventregistry.Info import *
from eventregistry.QueryArticles import *
from eventregistry.Counts import *


class GetTopCorrelations(QueryParamsBase):
    def __init__(self,
                 eventRegistry):    # instance of EventRegistry class
        QueryParamsBase.__init__(self)
        self._er = eventRegistry
        self._setVal("action", "findTopCorrelations")


    def _getPath(self):
        return "/json/correlate"

    #
    # input data methods
    #
    def setCustomInputData(self, inputDataArr):
        """
        specify the user defined array of input data

        @param inputDataArr: array of tuples (date, val) where date is a date object or string in YYYY-MM-DD format
                and val is the value/counts for that date
        """
        # clear any past test data values
        self._clearVal("testData")
        for (date, val) in inputDataArr:
            assert isinstance(val, (int, float)), "Value is expected to be a number"
            dateStr = None
            if isinstance(val, datetime.datetime):
                dateStr = val.date().isoformat()
            elif isinstance(val, datetime.date):
                dateStr = val.isoformat()
            elif isinstance(val, six.string_types):
                assert re.match("\d{4}-\d{2}-\d{2}", date)
                dateStr = date
            else:
                assert False, "First argument in the tuple is not a valid date"
            self._addArrayVal("testData", {"date": dateStr, "count": val})


    def loadInputDataWithQuery(self, queryArticles):
        """
        use the queryArticles to find articles that match the criteria. For the articles that match
        criteria in queryArticles compute the time-series (number of resulting articles for each date)
        an use the time series as the input data

        @param queryArticles: an instance of QueryArticles class, containing the conditions that are use to
            find the matching time-series. You don't need to specify any requested result.
        """
        # clear any past test data values
        self._clearVal("testData")

        assert isinstance(queryArticles, QueryArticles), "'queryArticles' excpected to be an instance of QueryArticles"
        queryArticles.setRequestedResult(RequestArticlesTimeAggr())
        res = self._er.execQuery(queryArticles)
        if "timeAggr" in res:
            for obj in res["timeAggr"]:
                self._addArrayVal("testData", json.dumps(obj))


    def loadInputDataWithCounts(self, getCounts):
        """
        use GetCounts class to obtain daily counts information for concept/category of interest
        @param getCounts: an instance of GetCounts class
        """
        # clear any past test data values
        self._clearVal("testData")

        assert isinstance(getCounts, GetCounts), "'getCounts' is expected to be an instance of GetCounts"
        res = self._er.execQuery(getCounts)
        assert len(list(res.keys())) <= 1, "The returned object had multiple keys. When creating the GetCounts instance use only one uri."
        assert len(list(res.keys())) != 0, "Obtained an empty object"
        assert "error" not in res, res.get("error")
        key = list(res.keys())[0]
        assert isinstance(res[key], list), "Expected a list"
        for obj in res[key]:
            self._addArrayVal("testData", json.dumps(obj))


    def hasValidInputData(self):
        """do we have valid input data (needed before we can compute correlations)"""
        return self._hasVal("testData")


    #
    # computing correlations
    #
    def getTopConceptCorrelations(self,
            candidateConceptsQuery = None,
            candidatesPerType = 1000,
            conceptType = None,
            exactCount = 10,
            approxCount = 0,
            returnInfo = ReturnInfo()):
        """
        compute concepts that correlate the most with the input data. If candidateConceptsQuery is provided we first identify the
        concepts that are potentially returned as top correlations. Candidates are obtained by making the query and analyzing the
        concepts that appear in the resulting articles. The top concepts are used as candidates among which we return the top correlations.
        If conceptType is provided then only concepts of the specified type can be provided as the result.

        @param candidateConceptsQuery: optional. An instance of QueryArticles that can be used to limit the space of concept candidates
        @param candidatesPerType: If candidateConceptsQuery is provided, then this number of concepts for each valid type will be return as candidates
        @param conceptType: optional. A string or an array containing the concept types that are valid candidates on which to compute top correlations
            valid values are "person", "org", "loc" and/or "wiki"
        @param exactCount: the number of returned concepts for which the exact value of the correlation is computed
        @param approxCount: the number of returned concepts for which only an approximate value of the correlation is computed
        @param returnInfo: specifies the details about the concepts that should be returned in the output result
        """

        self._clearVal("contextConceptIds")

        # generate all necessary parameters (but don't update the params of the self)
        params = QueryParamsBase.copy(self)

        # compute the candidates
        if candidateConceptsQuery != None:
            assert isinstance(candidateConceptsQuery, QueryArticles), "'candidateConceptsQuery' is expected to be of type QueryArticles"
            candidateConceptsQuery.setRequestedResult(RequestArticlesConceptAggr())
            candidateConceptsQuery._setVal("conceptAggrConceptCountPerType", candidatesPerType)
            candidateConceptsQuery._setVal("conceptAggrConceptIdOnly", True)
            ret = self._er.execQuery(candidateConceptsQuery)
            if ret and "conceptAggr" in ret:
                params._setVal("contextConceptIds", ",".join([str(x) for x in ret["conceptAggr"]]))
            else:
                print("Warning: Failed to compute a candidate set of concepts")

        if conceptType:
            params._setVal("conceptType", conceptType)
        params._setVal("exactCount", exactCount)
        params._setVal("approxCount", approxCount)
        params._setVal("sourceType", "news-concept")

        #
        # compute the correlations
        ret = self._er.jsonRequest(self._getPath(), params.queryParams)

        #
        # extend the return information with the details about the concepts (label, ...)
        if returnInfo != None:
            conceptIds = []
            if ret and ret["news-concept"]["exactCorrelations"]:
                conceptIds += [info["id"] for info in ret["news-concept"]["exactCorrelations"]]
            if ret and ret["news-concept"]["approximateCorrelations"]:
                conceptIds += [info["id"] for info in ret["news-concept"]["approximateCorrelations"]]
            conceptInfos = {}
            for i in range(0, len(conceptIds), 500):
                ids = conceptIds[i:i+500]
                q = GetConceptInfo(returnInfo = returnInfo)
                q.queryById(ids)
                info = self._er.execQuery(q)
                conceptInfos.update(info)
            if ret and ret["news-concept"]["exactCorrelations"]:
                for item in ret["news-concept"]["exactCorrelations"]:
                    item["conceptInfo"] = conceptInfos.get(str(item["id"]), {})
            if ret and ret["news-concept"]["approximateCorrelations"]:
                for item in ret["news-concept"]["approximateCorrelations"]:
                    item["conceptInfo"] = conceptInfos.get(str(item["id"]), {})

        # return result
        return ret


    def getTopCategoryCorrelations(self,
            exactCount = 10,
            approxCount = 0,
            returnInfo = ReturnInfo()):
        """
        compute categories that correlate the most with the input data.

        @param exactCount: the number of returned categories for which the exact value of the correlation is computed
        @param approxCount: the number of returned categories for which only an approximate value of the correlation is computed
        @param returnInfo: specifies the details about the categories that should be returned in the output result
        """

        # generate all necessary parameters (but don't update the params of the self)
        params = QueryParamsBase.copy(self)
        # don't send unnecessary data
        params._clearVal("contextConceptIds")
        params._setVal("exactCount", exactCount)
        params._setVal("approxCount", approxCount)
        params._setVal("sourceType", "news-category")

        #
        # compute the correlations
        ret = self._er.jsonRequest(self._getPath(), params.queryParams)

        #
        # extend the return information with the details about the categories (label, ...)
        if returnInfo != None:
            categoryIds = []
            if ret and ret["news-category"]["exactCorrelations"]:
                categoryIds += [info["id"] for info in ret["news-category"]["exactCorrelations"]]
            if ret and ret["news-category"]["approximateCorrelations"]:
                categoryIds += [info["id"] for info in ret["news-category"]["approximateCorrelations"]]
            categoryInfos = {}
            for i in range(0, len(categoryIds), 500):
                ids = categoryIds[i:i+500]
                q = GetCategoryInfo(returnInfo = returnInfo)
                q.queryById(ids)
                info = self._er.execQuery(q)
                categoryInfos.update(info)
            if ret and ret["news-category"]["exactCorrelations"]:
                for item in ret["news-category"]["exactCorrelations"]:
                    item["categoryInfo"] = categoryInfos.get(str(item["id"]), {})
            if ret and ret["news-category"]["approximateCorrelations"]:
                for item in ret["news-category"]["approximateCorrelations"]:
                    item["categoryInfo"] = categoryInfos.get(str(item["id"]), {})

        # return result
        return ret



from setuptools import setup
import re

VERSIONFILE = "eventregistry/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

def readme():
    with open('README.md') as f:
        return f.read()

setup(name='eventregistry',
      version = verstr,
      description = "A package that can be used to query information in Event Registry (http://eventregistry.org/)",
      classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Text Processing :: General',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Intended Audience :: Information Technology'
      ],
      url = 'https://github.com/EventRegistry/event-registry-python',
      author = 'Gregor Leban',
      author_email = 'gregor@eventregistry.org',
      license = 'MIT',
      packages = ['eventregistry'],
      install_requires = [
          'requests', 'six', 'pytz'
      ],
      zip_safe=False)



from eventregistry.Base import *
from eventregistry.ReturnInfo import *


class GetSourceInfo(QueryParamsBase):
    def __init__(self,
                 uriOrUriList = None,
                 returnInfo = ReturnInfo()):
        """
        obtain desired information about one or more news sources
        @param uriOrUriList: single source uri or a list of source uris for which to return information
        @param returnInfo: what details about the source should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getInfo")
        if uriOrUriList != None:
            self.queryByUri(uriOrUriList)
        self._update(returnInfo.getParams())


    def queryByUri(self, uriOrUriList):
        """search sources by uri(s)"""
        self._setVal("uri", uriOrUriList)


    def queryById(self, idOrIdList):
        """search concepts by id(s)"""
        self._setVal("id", idOrIdList)


    def _getPath(self):
        return "/json/source"



class GetConceptInfo(QueryParamsBase):
    def __init__(self,
                 uriOrUriList = None,
                 returnInfo = ReturnInfo()):
        """
        obtain information about concepts
        @param uriOrUriList: single concept uri or a list of concept uris for which to return information
        @param returnInfo: what details about the source should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getInfo")
        if uriOrUriList != None:
            self.queryByUri(uriOrUriList)
        self._update(returnInfo.getParams())


    def queryByUri(self, uriOrUriList):
        """search concepts by uri(s)"""
        self._setVal("uri", uriOrUriList)


    def _getPath(self):
        return "/json/concept"



class GetCategoryInfo(QueryParamsBase):
    def __init__(self,
                 uriOrUriList = None,
                 returnInfo = ReturnInfo()):
        """
        obtain information about categories
        @param uriOrUriList: single category uri or a list of category uris for which to return information
        @param returnInfo: what details about the source should be included in the returned information
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getInfo")
        if uriOrUriList != None:
            self.queryByUri(uriOrUriList)
        self._update(returnInfo.getParams())


    def queryByUri(self, uriOrUriList):
        """search categories by their uri(s)"""
        self._setVal("uri", uriOrUriList)


    def _getPath(self):
        return "/json/category"



class GetSourceStats(QueryParamsBase):
    def __init__(self, sourceUri = None):
        """
        get stats about one or more sources - return json object will include:
         "uri"
         "id"
         "totalArticles" - total number of articles from this source
         "withStory" - number of articles assigned to a story (cluster)
         "duplicates" - number of articles that are duplicates of another article
         "duplicatesFromSameSource" - number of articles that are copies from articles
            from the same source (not true duplicates, just updates of own articles)
         "dailyCounts" - json object with date as the key and number of articles for that day as the value
        """
        QueryParamsBase.__init__(self)
        self._setVal("action", "getStats")
        if sourceUri:
            self._setVal("uri", sourceUri)


    def _getPath(self):
        return "/json/source"


    def queryByUri(self, uriOrUriList):
        """ get stats about one or more sources specified by their uris """
        self.queryParams["uri"] = uriOrUriList;
